<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2010-02-23
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'core/om/BaseTagEntryPeer.php';

// The object class
include_once 'core/TagEntry.php';


/** 
 * The skeleton for this class was autogenerated by Propel  *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core 
 */
class TagEntryPeer extends BaseTagEntryPeer {
	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	TagEntry
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, TagEntryPeer::$_pkCache)) {
			return TagEntryPeer::$_pkCache[$pk];
		}
		$pk_array = explode('-', $pk);
		$object = parent::retrieveByPk($pk_array[0],$pk_array[1],$pk_array[2], $con);
		if ($useCache) {
			TagEntryPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, TagEntryPeer::$_pkCache)) {
			unset(TagEntryPeer::$_pkCache[$pk]);
		}
	}

	static function userCanUpdate($section, $ref_id){
		$item = self::getItemToTag($section, $ref_id);
		if (!is_object($item)) {
			return false;
		}
		
		return $item->userCanUpdate();
	}

	static function userCanRead($section, $ref_id){
		$item = self::getItemToTag($section, $ref_id);
		if (!is_object($item)) {
			return false;
		}
		
		return $item->userCanRead();
	}

	static function getItemToTag($section, $ref_id){
		$item = null;
		switch ($section){
			case TrackerItem::SECTION:
				$item = TrackerItemPeer::retrieveByPK($ref_id);
				break;
			case DocmanFile::SECTION:
				$item = DocmanFilePeer::retrieveByPk($ref_id);
				break;
			case FrsRelease::SECTION:
				$item = FRSReleasePeer::retrieveByPk($ref_id);
				break;
			case WikiPage::SECTION:
				$item = WikiPagePeer::retrieveByPk($ref_id);
				break;
			case ForumMessage::SECTION:
				$item = ForumMessagePeer::retrieveByPk($ref_id);
				break;
			case ScmCommit::SECTION:
				$item = ScmCommitPeer::retrieveByPk($ref_id);
				break;
			default:
				throw new InvalidDataException(array("Invalid tag section: $section"));
		}
		
		return $item;
	}
	
	/**
	 * Add a list of space-separated tags to an item
	 */
	static function addTags($section, $ref_id, $tags) {
		$parsed_tags = self::parseTags($tags);

		foreach ($parsed_tags as $tag) {
			$TagEntry = new TagEntry();
			$TagEntry->setRefId($ref_id);
			$TagEntry->setSection($section);
			$TagEntry->setTagName($tag);
			try{
				$TagEntry->save();
			} catch (Exception $ex){
				// This happens when there's a dupicate tag -- ignore the exception
			}	
		}
	}
	
	static function deleteTags($section, $ref_id, $tags) {
		$parsed_tags = self::parseTags($tags);

		foreach ($parsed_tags as $tag) {
			$c = new Criteria();
			$c->add(self::SECTION, $section);
			$c->add(self::REF_ID, $ref_id);
			$c->add(self::TAG_NAME, $tag);
			self::doDelete($c);
		}		
	}
	
	private static function parseTags($tags) {
		// Cleanup extra spaces, separe words by spaces
		$tags = preg_replace('/\s+/', ' ', $tags);
		$tag_array = explode(' ',$tags);
		$parsed_tags = array();

		foreach($tag_array as $tag){
			$parsed_tags[] = trim(mb_strtolower(Validator::sanitizeString($tag), 'UTF-8'));
		}
		
		return $parsed_tags;

	}
}
?>
