<?php 

require_once 'propel/om/BaseObject.php';

 
require_once 'propel/om/Persistent.php';

 

include_once 'propel/util/Criteria.php';

include_once 'core/ActivityLogPeer.php';

/**
 * Base class that represents a row from the 'activity_log' table.
 *
 *  
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to ActivityLog class. 
 * 
 * @package core 
 */
abstract class BaseActivityLog extends BaseObject implements Persistent {
	
	/** 
	 * The Peer class.
	 * Instance provides a convenient way of calling static methods on a class
	 * that calling code may not be able to identify.
	 * @var ActivityLogPeer
	 */
	protected static $peer;

	/**
	 * The value for the activity_date field.
	 * @var int	  
	 */
	protected $activity_date = -1;	

	/**
	 * The value for the project_id field.
	 * @var int	  
	 */
	protected $project_id = 0;	

	/**
	 * The value for the page field.
	 * @var string	  
	 */
	protected $page;	

	/**
	 * The value for the activity_type field.
	 * @var int	  
	 */
	protected $activity_type = 0;	

	/**
	 * Get the [optionally formatted] `activity_date` column value.
	 *  
	 * @param string $format The date/time format string (either date()-style or strftime()-style).
	 *							If format is NULL, then the integer unix timestamp will be returned.
	 * @return mixed Formatted date/time value as string or integer unix timestamp (if format is NULL).
	 * @throws PropelException - if unable to convert the date/time to timestamp.
	 */
	public function getActivityDate($format = 'Y-m-d H:i:s')
	{
		if ($this->activity_date === null || $this->activity_date === '') {
			return null;
		} elseif (!is_int($this->activity_date)) {
			// a non-timestamp value was set externally, so we convert it
			$ts = strtotime($this->activity_date);
			if ($ts === -1) {
				throw new PropelException("Unable to parse value of activity_date as date/time value: " . var_export($this->activity_date, true));
			}
		} else {
			$ts = $this->activity_date;
		}
		if ($format === null) {
			return $ts;
		} elseif (strpos($format, '%') !== false) {
			return strftime($format, $ts);
		} else {
			return date($format, $ts);
		}
	}


	/**
	 * Set the value of `activity_date` column.	  
	 *  
	 * @param int $v new value
	 * @return void
	 * @throws PropelException - If passed [not-null] date/time is in an invalid format. 
	 */
	public function setActivityDate($v)
	{
		if ($v !== null && !is_int($v)) {
			$ts = strtotime($v);
			if ($ts === -1) {
				throw new PropelException("Unable to parse date/time value for activity_date from input: " . var_export($v, true));
			}
		} else {
			$ts = $v;
		}
		if ($this->activity_date !== $ts || $ts === -1) {
			$this->activity_date = $ts;
			$this->modifiedColumns[] = ActivityLogPeer::ACTIVITY_DATE;
		}
		
	}

  
	/**
	 * Get the ProjectId column value.
	 *  
	 * @return int	  
	 */
	public function getProjectId()
	{
		return $this->project_id;
	}


	/**
	 * Set the value of `project_id` column.	  
	 *  
	 * @param int $v new value
	 * @return void
	 *  
	 */
	public function setProjectId($v)
	{
		if ($this->project_id !== $v || $v === 0) {
			$this->project_id = $v;
			$this->modifiedColumns[] = ActivityLogPeer::PROJECT_ID;
		}
				
	}

  
	/**
	 * Get the Page column value.
	 *  
	 * @return string	  
	 */
	public function getPage()
	{
		return $this->page;
	}


	/**
	 * Set the value of `page` column.	  
	 *  
	 * @param string $v new value
	 * @return void
	 *  
	 */
	public function setPage($v)
	{
		if ($this->page !== $v) {
			$this->page = $v;
			$this->modifiedColumns[] = ActivityLogPeer::PAGE;
		}
				
	}

  
	/**
	 * Get the ActivityType column value.
	 *  
	 * @return int	  
	 */
	public function getActivityType()
	{
		return $this->activity_type;
	}


	/**
	 * Set the value of `activity_type` column.	  
	 *  
	 * @param int $v new value
	 * @return void
	 *  
	 */
	public function setActivityType($v)
	{
		if ($this->activity_type !== $v || $v === 0) {
			$this->activity_type = $v;
			$this->modifiedColumns[] = ActivityLogPeer::ACTIVITY_TYPE;
		}
				
	}


	
	/**
	 * Hydrates (populates) the object variables with values from the database resultset.
	 *
	 * An offset (1-based "start column") is specified so that objects can be hydrated
	 * with a subset of the columns in the resultset rows.  This is needed, for example,
	 * for results of JOIN queries where the resultset row includes columns from two or
	 * more tables.
	 *
	 * @param ResultSet $rs The ResultSet class with cursor advanced to desired record pos.
	 * @param int $startcol 1-based offset column which indicates which restultset column to start with.
	 * @return void
	 * @throws PropelException  - Any caught Exception will be rewrapped as a PropelException.
	 */
	public function hydrate(ResultSet $rs, $startcol = 1)
	{
		try {
			$this->activity_date = $rs->getTimestamp($startcol + 0, null);
			$this->project_id = $rs->getInt($startcol + 1);					
			$this->page = $rs->getString($startcol + 2);					
			$this->activity_type = $rs->getInt($startcol + 3);					
			 
			$this->resetModified();
			$this->setNew(false);
		
		} catch (Exception $e) {
			throw new PropelException("Error populating ActivityLog object", $e);
		}		
	
	}
	
	/**
	 * Builds a Criteria object containing the primary key for this object.
	 *
	 * Unlike buildCriteria() this method includes the primary key values regardless
	 * of whether or not they have been modified.
	 * 
	 * @return Criteria The Criteria object containing value(s) for primary key(s).
	 */
	public function buildPkeyCriteria()
	{		
		$criteria = new Criteria(ActivityLogPeer::DATABASE_NAME);
		return $criteria;			
	}
	
	/**
	 * Build a Criteria object containing the values of all modified columns in this object.
	 *
	 * @return Criteria The Criteria object containing all modified values.
	 */
	public function buildCriteria()
	{		
		$criteria = new Criteria(ActivityLogPeer::DATABASE_NAME);
		if ($this->isColumnModified(ActivityLogPeer::ACTIVITY_DATE)) $criteria->add(ActivityLogPeer::ACTIVITY_DATE, $this->activity_date);
		if ($this->isColumnModified(ActivityLogPeer::PROJECT_ID)) $criteria->add(ActivityLogPeer::PROJECT_ID, $this->project_id);
		if ($this->isColumnModified(ActivityLogPeer::PAGE)) $criteria->add(ActivityLogPeer::PAGE, $this->page);
		if ($this->isColumnModified(ActivityLogPeer::ACTIVITY_TYPE)) $criteria->add(ActivityLogPeer::ACTIVITY_TYPE, $this->activity_type);
		return $criteria;			
	}
	
	

	private $fieldNames;

	/**
	 * Generate a list of field names.
	 *
	 * @return array A list of field names
	 */
	public function getFieldNames()
	{
		if ($this->fieldNames === null) {
			$this->fieldNames = array("activity_date", "project_id", "page", "activity_type", );
		}
		return $this->fieldNames;
	}
	
	/**
	 * Retrieves a field from the object by name passed in as a string.
	 * The string must be one of the static strings defined in this Class' Peer.
	 *
	 * @param string $name peer name
	 * @return mixed Value of field.
	 */
	public function getByName($name)
	{
		switch($name) {
	  
			case ActivityLogPeer::ACTIVITY_DATE:
				return $this->getActivityDate();
				break;
		  
			case ActivityLogPeer::PROJECT_ID:
				return $this->getProjectId();
				break;
		  
			case ActivityLogPeer::PAGE:
				return $this->getPage();
				break;
		  
			case ActivityLogPeer::ACTIVITY_TYPE:
				return $this->getActivityType();
				break;
	  
			default:
				return null;
				
		} // switch()
	}

	/**
	 * Retrieves a field from the object by Position as specified in the xml schema. 
	 * Zero-based.
	 *
	 * @param int $pos position in xml schema
	 * @return mixed Value of field at $pos
	 */
	public function getByPosition($pos)
	{
		switch($pos) {
  
			case 0:
				return $this->getActivityDate();
				break;
  
			case 1:
				return $this->getProjectId();
				break;
  
			case 2:
				return $this->getPage();
				break;
  
			case 3:
				return $this->getActivityType();
				break;
			default:
				return null;
		} // switch()
	}



	
	/**
	 * Convenience method to populate the object using an array.
	 * This is particularly useful when populating an object from one of the request arrays (e.g. $_POST).  This method
	 * goest through the column (peer) names, checking to see whether a matching key exists in populated array. If
	 * so the setByName() method is called for that column.
	 * @param array $arr An array where keys are column names.
	 * @return void
	 */
	public function populateFromArray($arr)
	{
  
		if (array_key_exists(ActivityLogPeer::ACTIVITY_DATE, $arr)) $this->setActivityDate($arr[ActivityLogPeer::ACTIVITY_DATE]);
  
		if (array_key_exists(ActivityLogPeer::PROJECT_ID, $arr)) $this->setProjectId($arr[ActivityLogPeer::PROJECT_ID]);
  
		if (array_key_exists(ActivityLogPeer::PAGE, $arr)) $this->setPage($arr[ActivityLogPeer::PAGE]);
  
		if (array_key_exists(ActivityLogPeer::ACTIVITY_TYPE, $arr)) $this->setActivityType($arr[ActivityLogPeer::ACTIVITY_TYPE]);
 
	}
	
	/**
	 * Sets a field from the object by name passed in as a string.
	 * The string must be one of the static strings defined in this Class' Peer.
	 *
	 * @param string $name peer name
	 * @param mixed $value field value
	 * @return void
	 */
	public function setByName($name, $value)
	{
		switch($name) {
	  
			case ActivityLogPeer::ACTIVITY_DATE:
				$this->setActivityDate($value);
				break;
	  
			case ActivityLogPeer::PROJECT_ID:
				$this->setProjectId($value);
				break;
	  
			case ActivityLogPeer::PAGE:
				$this->setPage($value);
				break;
	  
			case ActivityLogPeer::ACTIVITY_TYPE:
				$this->setActivityType($value);
				break;
				  
		} // switch()
	}

	/**
	 * Sets a field from the object by Position as specified in the xml schema.
	 * Zero-based.
	 *
	 * @param int $pos position in xml schema
	 * @param mixed $value field value
	 * @return void
	 */
	public function setByPosition($pos, $value)
	{
		switch($pos) {
  
			case 0:
				$this->setActivityDate($value);
				break;
  
			case 1:
				$this->setProjectId($value);
				break;
  
			case 2:
				$this->setPage($value);
				break;
  
			case 3:
				$this->setActivityType($value);
				break;
		} // switch()
	}


 
	/**
	 * Removes this object from datastore and sets delete attribute.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see BaseObject::setDeleted()
	 * @see BaseObject::isDeleted()
	 */
	public function delete($con = null)
	{	
		if ($this->isDeleted()) {
			throw new PropelException("This object has already been deleted.");
		}
		
		if ($con === null)
			$con = Propel::getConnection(ActivityLogPeer::DATABASE_NAME);

		try {
			$con->begin();
			ActivityLogPeer::doDelete($this, $con);
			$this->setDeleted(true);
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}
	

	/**
	 * flag to prevent endless save loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var boolean
	 */
	private $alreadyInSave = false;

	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.  This method
	 * wraps the doSave() worker method in a transaction.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	public function save($con = null)
	{
		if ($this->isDeleted()) {
		//	throw new PropelException("You cannot save an object that has been deleted.");
			return;
		}
		
		if ($con === null)
			$con = Propel::getConnection(ActivityLogPeer::DATABASE_NAME);

		try {
			$con->begin();
			$this->doSave($con);			
			$con->commit();
		} catch (PropelException $e) {
			$con->rollback();
			throw $e;
		}
	}
  
	/**
	 * Stores the object in the database.  If the object is new,
	 * it inserts it; otherwise an update is performed.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
		protected function doSave($con)	{ 
		
	 
		if (!$this->alreadyInSave) {
			$this->alreadyInSave = true;

			// If this object has been modified, then save it to the database.
			if ($this->isModified()) {
				if ($this->isNew()) {
					$pk = ActivityLogPeer::doInsert($this, $con);
					$this->setNew(false);
				} else {
					ActivityLogPeer::doUpdate($this, $con);
				}			
				$this->resetModified(); // [HL] After being saved an object is no longer 'modified'
			}

			$this->alreadyInSave = false;
		}
	}

	/**
	 * Validates the objects modified field values.
	 * This includes all objects related to this table.
	 *
	 * If $columns is either a column name or an array of column names
	 * only those columns are validated.
	 *
	 * @param mixed $columns Column name or an array of column names.
	 *
	 * @return mixed <code>true</code> if all columns pass validation
	 *			  or an array of <code>ValidationFailed</code> objects for columns that fail.
	 */
	public function validate($columns = null)
	{
	  if ($columns)
	  {
		return ActivityLogPeer::doValidate($this, $columns);
	  }

		return $this->doValidate();
	}

	/**
	 * flag to prevent endless validation loop, if this object is referenced
	 * by another object which falls in this transaction.
	 * @var boolean
	 */
	protected $alreadyInValidation = false;
  
	/**
	 * This function performs the validation work for complex object models.
	 * 
	 * In addition to checking the current object, all related objects will
	 * also be validated.  If all pass then <code>true</code> is returned; otherwise
	 * an aggreagated array of ValidationFailed objects will be returned.
	 *
	 * @return mixed <code>true</code> if all validations pass; array of <code>ValidationFailed</code> objets otherwise.
	 */
	protected function doValidate()
	{
		if (! $this->alreadyInValidation) {
			$this->alreadyInValidation = true;
			$retval = null;
	  
			$failureMap = array();	  

			if (($retval = ActivityLogPeer::doValidate($this)) !== true) {
				$failureMap = array_merge($failureMap, $retval);
			}
	  
	  
			$this->alreadyInValidation = false;
		}
	
		return (!empty($failureMap) ? $failureMap : true);
	}
  

	
	/**
	 * Dummy primary key setter.
	 * For now this function needs to exist because it's mandated in the Persistent
	 * interface, and because other methods will attempt to set the primary key.
	 * 
	 * This should be removed in favor of more complex template work.
	 */
	 public function setPrimaryKey($pk)
	 {
		 // do nothing, because this doesn't support primary keys
	 }	 

	/**
	 * Returns an id that differentiates this object from others
	 * of its class.
	 * @return null 
	 */
	public function getPrimaryKey()
	{

		return null;

	}

	
	/**
	 * Makes a copy of this object that will be inserted as a new row in table when saved.
	 * It creates a new object filling in the simple attributes, but skipping any primary
	 * keys that are defined for the table.
	 *  
	 * If desired, this method can also make copies of all associated (fkey referrers)
	 * objects.
	 *
	 * @param boolean $deepCopy Whether to also copy all rows that refer (by fkey) to the current row.
	 *  
	 * @return ActivityLog Clone of current object.
	 * @throws PropelException
	 */
	public function copy($deepCopy = false) 
	{
		$copyObj = new ActivityLog();
		$copyObj->setActivityDate($this->activity_date);
		$copyObj->setProjectId($this->project_id);
		$copyObj->setPage($this->page);
		$copyObj->setActivityType($this->activity_type);
		
		$copyObj->setNew(true);
		

		return $copyObj;
	}
		

	/**
	 * returns a peer instance associated with this om.  Since Peer classes	
	 * are not to have any instance attributes, this method returns the
	 * same instance for all member of this class. The method could therefore
	 * be static, but this would prevent one from overriding the behavior.
	 * @return ActivityLogPeer
	 */
	public function getPeer()
	{
		if (self::$peer === null) {
			self::$peer = new ActivityLogPeer();
		}
		return self::$peer;
	}

}
