<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli	<marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * This class handles the cron jobs. Note that this methods are called by root
 * so we have FULL system control. Be careful!
 */

require_once('propel/Propel.php');
require_once('SystemUtils.php');
require_once('initialize.php');
require_once('core/Plugin.php');
require_once('GFConf.php');
require_once('plugin/PluginFactory.php');
require_once('system/Cronjob.php');

class CronManager {
	public function __construct() {
		// Make sure the plugins are initialized
		$plugfact = PluginFactory::getFactory();
		$plugfact->initializePlugins();
	}
	
	public function setCronjobParameters($parameters) {
		Cronjob::setGlobalParameters($parameters);
	}
	
	/**
	 * Process all the cronjobs
	 */
	public function processCrons($cronexp='') {
		// Build the array of cronjobs to execute
		$crons = array();
		$cronpath = SystemUtils::buildFilePath(GFConf::get('system.libDir'), '/cronjobs');
		
		// Switch to a special "cronjob" session
		$session = Session::getSession();
		$session->beginCronjobSession();

		$dh = opendir($cronpath);
		while ($cronfile = readdir($dh)) {
			if (!preg_match("/\\.php\$/i", $cronfile)) {
				// Not a PHP file
				continue;
			}
			
			$fullcron = SystemUtils::buildFilePath($cronpath, $cronfile);
			// run only cronjobs matching regexp
			if (!empty($cronexp) && !preg_match('/'.$cronexp.'/', $fullcron)) {
				continue;
			}

			$crons[] = $fullcron;
		}
		
		// now look for the plugin's cronjobs
		$plugins = PluginPeer::doSelect(new Criteria());
		$pluginPath = GFConf::get('system.pluginsPath');
		foreach ($plugins as $plugin) {
			$pluginCronPath = SystemUtils::buildFilePath($pluginPath, '/'.$plugin->getPluginName().'/cronjobs');
			if (is_dir($pluginCronPath)) {
				$dh = opendir($pluginCronPath);
				while ($cronfile = readdir($dh)) {
					if (!preg_match("/\\.php\$/i", $cronfile)) {
						// Not a PHP file
						continue;
					}
					
					$fullcron = SystemUtils::buildFilePath($pluginCronPath, $cronfile);
					// run only cronjobs matching regexp
					if (!empty($cronexp) && !preg_match('/'.$cronexp.'/', $fullcron)) {
						continue;
					}
					
					// Special case -- don't run "external" cronjobs
					if (strpos($cronfile, "external") === 0) {
						if ($output) {
							echo "Skipping external cronjob $fullcron...\n";
						}
						continue;
					}

					$crons[] = $fullcron;
				}
			}
		}
		
		foreach ($crons as $cron) {
			$this->_run($cron);
		}
	}
	
	/**
	 * Run a particular cronjob
	 */
	public function processCron($cronjob, $output) {
		// Switch to a special "cronjob" session
		$session = Session::getSession();
		$session->beginCronjobSession();

		$this->_run($cronjob, $output);
	}
	
	private function _run($cronjob, $output=true) {
		if ($output) {
			echo "Running $cronjob...\n";
		}
		chdir(dirname($cronjob));
		
		if ($output) {
			include($cronjob);
		} else {
			ob_start();
			include($cronjob);
			ob_end_clean();
		}
	}
}
?>