<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/news/lib/om/BaseNews.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once('SiteNotification.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.news.lib 
 */

require_once('notification/NotificationManager.php');
require_once('plugins/news/lib/NewsNotification.php');

class News extends BaseNews implements GFPermission {
	const SECTION = 'news';
	
	private $invalid_data = array();
	
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see News::setDeleted()
	 * @see News::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete News object: permission denied");
		}
		ActionManager::notify("NEWS_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create News object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update News object: permission denied");
		}
		if (count($this->invalid_data) > 0) {
			throw new InvalidDataException($this->invalid_data);
		}
		
		$notify = $this->isNew();		// notification must be called after we save the object

		parent::save($con);
		ActionManager::notify("NEWS_SAVE", $this);
		
		if ($notify) {
			$notification = new NewsNotification($this);
			NotificationManager::queue($notification);
		}
	}
	
	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
	    $project = $this->getProject();
	    if ($project->userIsAdmin()){
			return true;
	    }
	    return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		$project = $this->getProject();
		if ($project->isPublic()) {
			return true;
		} else if (Session::getSession()->isLoggedIn() && Session::getSession()->getUser()->isProjectMember($project)) {
			return true;
		}

		return false;
	}

	
	/**
	 * Retrieve a list of the users that expect to be notified of the news
	 * NOTE: the ref_id in the monitor section refers to the project ID, not a
	 * particular news_id.
	 */
	public function getMonitoringUsers() {
		return MonitorPeer::getMonitoringUsers(self::SECTION, $this->getProject()->getPrimaryKey());
	}
	
	/**
	 * This static method tells wheter a user is monitoring a project's news. It _must_
	 * be static because it is not linked to a particular news.
	 * @param	User $user
	 * @param	Project $project
	 */
	public static function userIsMonitoringProjectNews(User $user, Project $project) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $project->getPrimaryKey());
		$c->add(MonitorPeer::USER_ID, $user->getPrimaryKey());
		$mon = MonitorPeer::doSelectOne($c);
		return (!is_null($mon)) ? true : false;
	}

	public function getShortDetails() {
		if (strlen($this->getDetails()) > 400) {
			return substr($this->getDetails(),0,400).'...';
		} else {
			return $this->getDetails();
		}
	}
	
	public function setSummary($summary) {
		if (strlen(trim($summary)) == 0) {
			$this->invalid_data['summary'] = Language::getSessText('News.SummaryInvalid');
			return;
		}

		parent::setSummary($summary);
		unset($this->invalid_data['summary']);
	}
	
	public function setDetails($details) {
		if (strlen(trim($details)) == 0) {
			$this->invalid_data['details'] = Language::getSessText('News.DetailsInvalid');
			return;
		}

		parent::setDetails($details);
		unset($this->invalid_data['details']);
	}
}
?>