<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id: post_commit.php 4537 2010-08-24 13:11:27Z tperdue $
 * @author Daniel A. Perez daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('initialize.php');

require_once('GFConf.php');
require_once('plugins/scm/lib/ScmCommitWrapper.php');
require_once('plugins/scmsvn/lib/SVNRepository.php');
require_once('GFUrl.php');

/**
 * PostSvnCommit - post the new committed files info to newsvncommit
 *
 */
class PostSvnCommit {
	
	private $debug,$debug_file; // debugging vars
	private $use_ssl,$default_domain,$svnpath;
	private $argv,$argc;
	
	/**
	 * Constructor
	 *
	 * Initialization of variables
	 * 
	 */	
	function __construct($argc,$argv) {
		$this->debug = GFConf::get('scmsvn.debug');
		$this->debug_file = GFConf::get('scmsvn.debugFile');
		$this->use_ssl = GFConf::get('system.use_ssl');
		$this->svnpath = GFConf::get('scmsvn.svnpath');
		$this->default_domain = GFConf::get('system.hostName');
		$this->argc = $argc;
		$this->argv = $argv;
		
		if ($this->debug) {
			$name = $this->debug_file;
			$this->debug_file = fopen($this->debug_file,"a+");	
			chmod($name,0666);
		}
	}

	/**
	 * Main procedure
	 */
	function Run() {
		if (count($this->argv) != 3) {
			echo "Usage: $this->argv[0] <repository> <revision>\n";
			echo "This program should be automatically called by SVN\n";
			exit;
		}

		if ($this->debug) {
			echo "Arguments count: ".$this->argc."\n";
			echo "Arguments passed to Post.php:\n";
			print_r($this->argv);
		}
		
		$files = array();
		$repositoryPath = $this->argv[1];
		$revision   = $this->argv[2];
		$output1 = array();
		$output2 = array();
		$output3 = array();
		
		$svnlook = SystemUtils::buildFilePath($this->svnpath, 'svnlook');
		
		// Fetch the user name, commit log and list of changed files
		SystemUtils::execute('LC_ALL="en_US.UTF-8" '.$svnlook.' author -r '.escapeshellarg($revision).' '.escapeshellarg($repositoryPath), $output1);
		$username = $output1[0];
		SystemUtils::execute('LC_ALL="en_US.UTF-8" '.$svnlook.' log -r '.escapeshellarg($revision).' '.escapeshellarg($repositoryPath), $output2);
		$log = implode("\n", $output2);
		SystemUtils::execute('LC_ALL="en_US.UTF-8" '.$svnlook.' changed -r '.escapeshellarg($revision).' '.escapeshellarg($repositoryPath).' | sed \'s/^[_ADU][ U]  //\'', $output3);
		$changed = $output3;

		if ($this->debug) {
			echo "Variables fetched:\n";
			echo "Username:  " . $username . " \n";
			echo "Log message:  " . $log . " \n";
			echo "Changed files:  " . $changed . " \n";
			fwrite($this->debug_file,"vars filled\n");
			fwrite($this->debug_file,"username :  " . $username . " \n");
			fwrite($this->debug_file,"log  :  " . $log . " \n");
			fwrite($this->debug_file,"changed :  " . $changed . " \n");
		}

		foreach ($changed as $onefile) {
			// Now try to see what was the last revision of this file before this commit
			$oldrev = $revision-1;	// Safe default
			$cmd = 'LC_ALL="en_US.UTF-8" '.$svnlook.' history -r '.escapeshellarg($revision).' '.escapeshellarg($repositoryPath).' '.escapeshellarg($onefile);
			$output = array();
			if (SystemUtils::execute($cmd, $output) == 0) {
				// $output is something like:
				//Array
				//(
				//    [0] => REVISION   PATH
				//    [1] => --------   ----
				//    [2] =>        6   /path/to/file
				//    [3] =>        5   /path/to/file
				//    [4] =>        1   /path/to/file
				//)
				if (count($output) <= 3) {
					// This means the file is new (only one revision)
					$oldrev = "none";
				} else {
					// Parse the line with the previous revision
					$oldrev_line = trim($output[3]);
					// Extract the revision number from the line
					preg_match('/^([0-9]*)/', $oldrev_line, $matches);
					$oldrev = $matches[1];
					echo $onefile." OLDREV: $oldrev\n";
				}
			} else {
				// Command had errors...
			}
			
			$files[] = array(
					'path' => '/'.$onefile,
					'prev_version' => $oldrev,
					'new_version' => $revision
				);
		}
		
		/* Generate the ScmCommitWrapper object */
		// Extract the project name (the last component of the path)
		$repositoryPath = preg_replace('/\\/*$/', '', $repositoryPath);		// remove trailing /
		preg_match('/([^\\/]*)$/', $repositoryPath, $matches);
		$project_name = $matches[1];
		
		$project = ProjectPeer::retrieveByUnixName($project_name);
		$repos = new SVNRepository($project);
		$user = UserPeer::retrieveByUnixName($username);
		
		$wrapper = new ScmCommitWrapper($repos, $user, $log, $files);
		$wrapper->generateCommitData();
		
		if ($this->debug) {
			fclose($this->debug_file);
		}
	}	
}

$poster = new PostSvnCommit($argc,$argv);
$poster->Run();

?>
