<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('core/Language.php');
require_once('TrackerPeer.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/TrackerExtraFieldPeer.php');
require_once('plugins/tracker/lib/TrackerExtraFieldElementPeer.php');
require_once('plugins/tracker/lib/TrackerItemScmCommit.php');
require_once('plugins/tracker/lib/TrackerCommitReassignTrigger.php');
require_once('plugins/tracker/lib/TrackerCommitTrigger.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerEfeTransitionReassign.php');
require_once('lib/GFSearch.php');
require_once('lib/core/ProjectPluginPeer.php');
require_once('lib/core/PluginPeer.php');
require_once('plugins/tracker/lib/TrackerItemNotification.php');
require_once('notification/NotificationManager.php');
require_once('lib/GFExportImportUtils.php');


class TrackerPlugin implements GFPlugin {

	//protected $project;
	CONST NOACCESS	= Tracker::PERMLEVEL_NONE;
	CONST READ = Tracker::PERMLEVEL_READ;
	CONST TECH = Tracker::PERMLEVEL_TECH;
	CONST TECHADMIN = Tracker::PERMLEVEL_TECH_ADMIN;
	CONST ADMIN = Tracker::PERMLEVEL_ADMIN;
	CONST NOANON = 0;
	CONST ANON = 1;
	CONST ISPRIVATE = 0;
	CONST ISPUBLIC = 1;	
	CONST SECTION = 'tracker';
	const PENDING_TASKS_SECTION = 'pending_tasks';

	/*function __construct($project) {
		$this->project = $project;
	}*/	
	function __construct() {
	}
	
	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	public function initialize() {
		ActionManager::registerListeners($this, $this->getListeners());
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}	
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'tracker';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	public function getListeners() {
		return array(
			'BUILD_PROJECT_TABS',
			'BUILD_PROJECT_SUMMARY',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
		//	'GET_TRACKER_NAV_ELEMENTS',
		//	'GET_PROJECT_REPORTING_NAV_ELEMENTS',
			'SCM_COMMIT',
			'SEARCH_BY_ID_LIST',
			'SEARCH_BY_ID_ACTION',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'BUILD_MYPAGE_MONITORS',
			'MONITOR_CHECK_PERMISSIONS',
			'CHECK_DOWNLOAD_PERMS',
			'MAIL_GATEWAY',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'GET_OBSERVER_ROLE_CHOICES',
			'UPDATE_OBSERVER_SETTINGS',
			'BUILD_RSS_FEED',
			'RENDER_ASSOCIATION_LINK',
			'GET_ASSOCIATED_ITEM_INFO',			
			'ASSOCIATION_SECTION',
			'ASSOCIATION_CHECK',
			'GET_DISK_USAGE',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'PROJECT_AUDIT_TRAIL',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
			'GET_PROJECT_SETTINGS_FORM',
			'DO_TAG_SEARCH'
			//'GET_LOCAL_NAV_ELEMENTS'
			);
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed
	 *	@return PluginResult
	 */
	public function handleAction($listenerName, $params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'BUILD_PROJECT_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
	/*		case 'GET_TRACKER_NAV_ELEMENTS': {
				return $this->getTrackerNavElements();
				break;
			}*/
			case 'GET_PROJECT_REPORTING_NAV_ELEMENTS': {
				return $this->getProjectReportingNavElements($params);
				break;
			}			
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'SCM_COMMIT': {
				return $this->scmCommit($params);
				break;
			}
			case 'SEARCH_BY_ID_LIST': {
				return $this->searchByIDList();
				break;
			}
			case 'SEARCH_BY_ID_ACTION': {
				return $this->searchByIDAction($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}				
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'CHECK_DOWNLOAD_PERMS': {
				return $this->checkDownloadPerms($params);
				break;
			}
			case 'MAIL_GATEWAY': {
				return $this->processMailGateway($params);
				break;
			}
			case 'GET_ROLE_CHOICES': {
				return $this->getRoleChoices($params);
				break;
			}
			case 'UPDATE_ROLE_CHOICES': {
				return $this->updateRoleChoices($params);
				break;
			}
			case 'GET_OBSERVER_ROLE_CHOICES': {
				return $this->getObserverRoleChoices($params);
				break;
			}
			case 'UPDATE_OBSERVER_SETTINGS': {
				return $this->updateObserverSettings($params);
				break;
			}
			case 'BUILD_RSS_FEED': {
				return $this->buildRssFeed($params);
				break;
			}
			case 'RENDER_ASSOCIATION_LINK': {
				return $this->renderAssociationLink($params);
				break;
			}
			case 'GET_ASSOCIATED_ITEM_INFO': {
				return $this->getAssociatedItemInfo($params);
				break;
			}
			case 'ASSOCIATION_SECTION': {
				return $this->getAssociationSection($params);
				break;
			}
			case 'ASSOCIATION_CHECK': {
				return $this->getAssociationCheck($params);
				break;
			}
			case 'GET_DISK_USAGE': {
				return $this->getDiskUsage($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'PROJECT_AUDIT_TRAIL': {
				return $this->getProjectAuditTrail($params);
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			case 'GET_PROJECT_SETTINGS_FORM': {
				return $this->getProjectSettingsForm($params);
				break;
			}
			case 'DO_TAG_SEARCH': {
				return $this->getSearchByTagsResult($params);
				break;
			}
			/*case 'GET_LOCAL_NAV_ELEMENTS': {
				return $this->getLocalNavElements($params);
				break;
			}*/
		}
	}


	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE 
							(
								(
									section='".TrackerPlugin::SECTION."' AND ref_id IN (
										SELECT tracker_id FROM tracker, project WHERE
										tracker.project_id = project.project_id
										AND
										project.project_id IN (".$project_ids.")
										AND
										NOT (project.is_public = true AND tracker.is_public = true)
									)
								)
								OR
								(
									section='".TrackerItem::SECTION."' AND ref_id IN
										(
										 SELECT tracker_item_id FROM tracker_item, tracker, project WHERE
										 tracker_item.tracker_id = tracker.tracker_id
										 AND
										 tracker.project_id = project.project_id
										 AND
										 project.project_id IN (".$project_ids.")
									   AND
										 NOT (project.is_public = true AND tracker.is_public = true)
										)
								)
							)								
							AND 
							user_id = ".$user_id;
			$stmt->executeQuery($sql);
		}		
	}



	private function deleteUserMonitors($user_project) {
		$user_id = $user_project->getUserId();
		$project = $user_project->getProject();
		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE section='".TrackerPlugin::SECTION."' AND ref_id IN (
									SELECT tracker_id as ref_id FROM tracker WHERE project_id=".$project->getProjectId()."
									)
								AND 
									user_id = ".$user_id;
		
				$stmt->executeQuery($sql);
			}
		}
	}

	private function getAssociationCheck($params) {
		if ($params->getSection() == TrackerItem::SECTION) {
			$ti=TrackerItemPeer::retrieveByPk($params->getRefId());
			if (!is_object($ti) || !$ti->userCanUpdate()) {
				throw new InvalidDataException(array('The tracker item doesn\'t exist or you don\'t have permissions for it'));
			}
		} 
		if ($params->getToSection() == TrackerItem::SECTION) {
			$ti=TrackerItemPeer::retrieveByPk($params->getToRefId());
			if (!is_object($ti) || !$ti->userCanUpdate()) {
				throw new InvalidDataException(array('The tracker item doesn\'t exist or you don\'t have permissions for it'));
			}
		}
	}

	private function getAssociationSection($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$result->setResult(array(TrackerItem::SECTION,'Tracker Item'));
		return $result;
	}

	private function renderAssociationLink($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==TrackerItem::SECTION) {
				$ti=TrackerItemPeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					$ret[]=array('Tracker Item: '.$ti->getTrackerItemLink(true),
					$param[2],
					$param[3]);
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;		
	}

	/**
	 *	Returns additional info for the associated item
	 *		 
	 *	@param array $params needed params to build the associated item info
	 * 	@return array with associated item info (Description, URL)
	 */	 			
	private function getAssociatedItemInfo($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==TrackerItem::SECTION) {
				$ti=TrackerItemPeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					$ret[]=array(
					$param[2],
					$param[3],
					$param[4],
					$param[5],
					$param[6],
					'Tracker Item: '.$ti->getSummary(),
					$ti->getTrackerItemUrl(false));
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;	
	}
		
	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(ForumPeer::DATABASE_NAME);
		$theme = $params['theme'];
		$today_arr = getdate();
		//$now = mktime($today_arr['hours'],$today_arr['minutes'],$today_arr['seconds'],$today_arr['mon'],$today_arr['mday'],$today_arr['year']);
		//$nowlater = mktime($today_arr['hours'],$today_arr['minutes'],$today_arr['seconds'],$today_arr['mon']+4,$today_arr['mday'],$today_arr['year']);
		//echo $nowlater;
		//get the assigned trackers for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();

		$sql = "SELECT t.tracker_id, t.tracker_name, p.project_id, p.project_name
		FROM tracker t, project p
		WHERE p.project_id = t.project_id 
		AND 
			(
				t.tracker_id IN (
					SELECT t.tracker_id
					FROM tracker t, monitor m
					WHERE m.user_id = '$user_id' 
					AND m.section='tracker'
					AND m.ref_id = t.tracker_id)
				OR
				t.tracker_id IN (
					SELECT t.tracker_id
					FROM tracker t, tracker_item ti, monitor m
					WHERE m.user_id = '$user_id' 
					AND m.section='trackeritem'
					AND t.tracker_id = ti.tracker_id
					AND m.ref_id = ti.tracker_item_id)
			)
		ORDER BY p.project_name ASC,  t.tracker_name ASC";

		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('Tracker.Monitoring').'</h3>';

		$theader = array();
		$theader[] = 'Project';
		$theader[] = Language::getSessText('My.ProjectTrackers');
		$last_project = null;
		if ($rs->getRecordCount()>0) {
			$theader[] = '';
			$value .= $theme->tableTop($theader);
			$ttype=array();
			while($rs->next()) {
				$ttype[$rs->getInt('project_id')][] = '1';
			}
			$rs->beforeFirst();	
			$rs->next();

			$rs->beforeFirst();	
			while($rs->next()) {
				$tracker = TrackerPeer::retrieveByPk( $rs->getInt('tracker_id') );
				$trow = array();
				$tmod = array();
				$project_name = $rs->getString('project_name');
				$tracker_name = $rs->getString('tracker_name');
				if ($last_project != $rs->getInt('project_id')) {
					$sql = "SELECT count(distinct(t.tracker_id)) as trackers_with_items
									FROM tracker_item ti, tracker t, monitor m
									WHERE m.user_id = '$user_id' AND m.section='".TrackerItem::SECTION."'
									AND m.ref_id=ti.tracker_item_id
									AND t.tracker_id = ti.tracker_id
									AND t.project_id = " . $rs->getInt('project_id') ;
									
					$rs_count = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
					$rs_count->next();
					$count_trackers = $rs_count->getInt('trackers_with_items');
					
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerBrowse',$this->getName(),'',array('tracker_id'=>$rs->getInt('tracker_id'))) . '" >' .  $project_name . '</a>';
					$tmod[]='rowspan="'.(count($ttype[$rs->getInt('project_id')])+$count_trackers).'" bgcolor="#FFFFFF" width="30%"';
				}
				
				$last_project = $rs->getString('project_id');				

				$sql = "SELECT *
							FROM tracker_item ti, monitor m
							WHERE m.user_id = '$user_id' AND m.section='".TrackerItem::SECTION."'
							AND m.ref_id=ti.tracker_item_id
							AND ti.tracker_id = " . $rs->getInt('tracker_id');
				$rs_inner = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
				$count_tracker_items = $rs_inner->getRecordCount();

				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerItemBrowse',$this->getName(),'',array('tracker_id'=>$rs->getInt('tracker_id'))) . '" >' .  $tracker_name . '</a>';
				if ($count_tracker_items > 0) {
					$tmod[] = '';
				} else {
					$tmod[] = '';
				}
				if ($tracker->userIsMonitoring($user)) {
					$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(Tracker::SECTION, $rs->getInt('tracker_id'), null, false).'" >'.Language::getSessText('TrackerItem.StopMonitor').'</a>';
				} else {
					$trow[] = '';
				}
				$tmod[] = 'align="right"';
				$value .= $theme->tableRow($trow, $tmod);

				if ($count_tracker_items > 0) {
					$trow = array();
					$tmod = array();

					$theader_inner = array();
					$theader_inner[] = 'Tracker Item';
					$theader_inner[] = '';
					$value_inner = $theme->tableTop($theader_inner);
								
					while($rs_inner->next()) {
						$trow_inner = array();
						$tmod_inner = array();
						$trow_inner[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerItemEdit',$this->getName(),'',array('tracker_item_id'=>$rs_inner->getString('tracker_item_id'))) . '" >' . $rs_inner->getString('summary') . '</a>';
						$tmod_inner[] = '';
					
						$trow_inner[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(TrackerItem::SECTION, $rs_inner->getInt('tracker_item_id'), null, false).'" >'.Language::getSessText('TrackerItem.StopMonitor').'</a>';;
						$tmod_inner[]='align="right"';
					
						$value_inner .= $theme->tableRow($trow_inner,$tmod_inner);
					}
				
					$value_inner .= $theme->tableBottom();
					$trow[] = $value_inner.'<br>';
					$tmod[]='colspan="2"';
					
					$value .= $theme->tableRow($trow, $tmod);		

					unset($trow);
					unset($mod);					
				} 
				

			}

	//exit;	
		} else {
			$value = $theme->tableTop($theader);
			$trow = array();
			$trow[] = Language::getSessText('My.NoMonitoredProjectTrackers');
			$tmod[]='colspan="2"';
			$value .= $theme->tableRow($trow,$tmod);
		}
	
		
		
	//	$value .= $theme->tableRow($trow, $tmod);
		
		$value .= $theme->tableBottom();
		
		// Pending tasks email
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::PENDING_TASKS_SECTION);
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		$monitor = MonitorPeer::doSelectOne($c);
		$is_monitoring = !is_null($monitor);
		
		$theader = array();
		$theader[] = Language::getSessText('TrackerPendingTask.Title');
		$theader[] = '';

		$value .= '<br />';
		$value .= $theme->tableTop($theader);
		
		$trow = array();
		if ($is_monitoring) {
			$trow[] = Language::getSessText('TrackerPendingTask.Enabled');
			$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(self::PENDING_TASKS_SECTION, '0', null, false).'" >'.Language::getSessText('TrackerPendingTask.Disable').'</a>';
		} else {
			$trow[] = Language::getSessText('TrackerPendingTask.Disabled');
			$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(self::PENDING_TASKS_SECTION, '0', null, true).'" >'.Language::getSessText('TrackerPendingTask.Enable').'</a>';
		}
		$tmod = array('width="30%"', 'align="right"');
		$value .= $theme->tableRow($trow, $tmod);
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}	
	
	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	
		$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);
		$theme = $params['theme'];
		
		// FIRST : get the assigned items
		$value = '
		<div class="tabbertab'.((isset($_GET['selectedtab']) && $_GET['selectedtab']=='assigned') ? ' tabbertabdefault' : '') .'" title="'. Language::getSessText('My.TrackerAssigned') . '">';
		//get the assigned items for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
		
		$sql = "select count(*) from tracker t, project p, tracker_item ti, tracker_item_assignee tia 
		where tia.assignee = '$user_id' 
		and ti.tracker_id = t.tracker_id 
		and tia.tracker_item_id = ti.tracker_item_id 
		and t.project_id = p.project_id
		and ti.status_id=".Tracker::STATUS_OPEN;
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$rs->next();
		$total = $rs->getInt(1);
		$columns=array('t.project_id,t.tracker_id' => Language::getSessText('Project.Project'),
			't.tracker_name'=>Language::getSessText('Tracker.TabString'),
			'ti.tracker_item_id'=>Language::getSessText('Common.Id'),
			'ti.priority'=>Language::getSessText('TrackerItem.PriorityName'),
			'ti.summary'=>Language::getSessText('TrackerItem.SummaryName')
		);
		$default_sort_by='t.project_id,t.tracker_id';
		$paginator = new Paginator('mytiassig', -1, $total, $default_sort_by, $columns);
		
		$sql = "select t.tracker_name, t.tracker_id, t.project_id, t.datatype, p.unix_name, p.project_name, ti.tracker_item_id, ti.summary, ti.priority, ti.details from tracker t, project p, tracker_item ti, tracker_item_assignee tia 
		where tia.assignee = '$user_id' 
		and ti.tracker_id = t.tracker_id 
		and tia.tracker_item_id = ti.tracker_item_id 
		and t.project_id = p.project_id
		and ti.status_id=".Tracker::STATUS_OPEN . $paginator->getSQL(); //" order by t.project_id,t.tracker_id desc";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		
		$last_tracker = null;
		$last_project = null;
		$last_type = -1;
		/*
		$theader = array();
		$theader[] = Language::getSessText('Project.Project');
		$theader[] = Language::getSessText('Tracker.TabString');
		$theader[] = Language::getSessText('Common.Id');
		$theader[] = Language::getSessText('TrackerItem.PriorityName');
		$theader[] = $paginator->insertLimitOptions(Language::getSessText('TrackerItem.SummaryName'));
		//$value .= $paginator->showLimitOptions();
		*/
		$sort = $paginator->getSortingOptions();
		$sort_by = $sort['sort_by'];
		$sort_order = $sort['order'];

		$theader = $paginator->getSortableTableHeaders($theme);

		$value .= $theme->tableTop($theader);
		
		$script = "<script type='text/javascript'>\n";
		
		if ($rs->getRecordCount()>0) {
			$ttype1=array();
			$ttype2=array();
			while($rs->next()) {
				$ttype1[$rs->getInt('tracker_id')][] = '1';
				$ttype2[$rs->getInt('project_id')][] = '1';
				// Javascript code for Drag & Drop
				$script .="new Draggable('jsTrackerItemId_".$rs->getInt('tracker_item_id')."', {revert:true,onStart:doUnblockLink,onDrag:doBlockLink});\n";
			}
			$rs->beforeFirst();			
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$tracker_name = $rs->getString('tracker_name');
				$project_name = $rs->getString('project_name');
				//if not using default sort, the grouping will get messed up
				if ($last_project != $rs->getString('project_id') || $default_sort_by != $sort_by) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id'))) . '" >' .  $project_name . '</a>';
					if ($default_sort_by != $sort_by) {
						$tmod[]='rowspan="1"';
					} else {
						$tmod[]='rowspan="'.count($ttype2[$rs->getInt('project_id')]).'" bgcolor="#FFFFFF"';
					}
				}				
				//if not using default sort, the grouping will get messed up
				if ($last_tracker != $rs->getString('tracker_id') || $default_sort_by != $sort_by) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerItemBrowse',$this->getName(),'',array('tracker_id'=>$rs->getInt('tracker_id'))) . '" >' .  $tracker_name . '</a>';
					if ($default_sort_by != $sort_by) {
						$tmod[]='rowspan="1"';
					} else {
						$tmod[]='rowspan="'.count($ttype1[$rs->getInt('tracker_id')]).'" bgcolor="#FFFFFF"';
					}
				}
				$last_project = $rs->getString('project_id');
				$trow[] = $rs->getInt('tracker_item_id');
				$tmod[]='';
				$trow[] = $rs->getString('priority');
				$tmod[]='bgcolor="'.$theme->getPriorityColor($rs->getString('priority')).'" id="jsPriorityItemId_'.$rs->getInt('tracker_item_id').'"';
				
				$last_tracker = $rs->getString('tracker_id');
				
				
				// Text that will be used on the tooltip
				$tooltip = StringUtils::trimString($rs->getString('details'), 500);
				$tooltip = StringUtils::generateTooltip($tooltip);
				
				$trow[] = '<div class="jsTrackerItems moveHandle" style="width:100%;" id="jsTrackerItemId_'.$rs->getInt('tracker_item_id').'"><a href="' . 
							GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerItemEdit',$this->getName(),'',
							array('tracker_id'=>$rs->getInt('tracker_id'),'tracker_item_id'=>$rs->getInt('tracker_item_id'))) . '" onmouseover="tooltip(\''.$tooltip.'\'); doUnblockLink();" 
							onmouseout="tooltipexit();" onclick="return linkWatch(this.href);">'.$rs->getString('summary').'</a></div>';
				$tmod[]='';
				
				$value .= $theme->tableRow($trow,$tmod);
			}
			//$value .= $theme->listPaginatorRow($paginator);
		} else {
			$value .= $theme->tableFullRow(Language::getSessText('My.NoAssignedTracker'));
		}
		$script .= "</script>\n";

		$value .= $theme->tableBottom();
		if(isset($paginator) && is_object($paginator)) {
			$value .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
		}

		$value .= '</div>';

		// Put the <script> part after the tables
		$value .= $script;
		
		// SECOND : get the items I submitted

		$value .= '
		<div class="tabbertab'.((isset($_GET['selectedtab']) && $_GET['selectedtab']=='submitted') ? ' tabbertabdefault' : '') .'" title="'. Language::getSessText('My.TrackerSubmitted') . '">';
		//get the submitted items by the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
		$sql = "select count(*) from tracker t, project p, tracker_item ti 
		where ti.submitted_by = '$user_id' 
		and ti.tracker_id = t.tracker_id 
		and t.project_id = p.project_id
		and ti.status_id=".Tracker::STATUS_OPEN;
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$rs->next();
		$total = $rs->getInt(1);
		
		$columns=array('t.project_id,t.tracker_id' => Language::getSessText('Project.Project'),
			't.tracker_name'=>Language::getSessText('Tracker.TabString'),
			'ti.tracker_item_id'=>Language::getSessText('Common.Id'),
			'ti.priority'=>Language::getSessText('TrackerItem.PriorityName'),
			'ti.summary'=>Language::getSessText('TrackerItem.SummaryName')
		);
		$default_sort_by='t.project_id,t.tracker_id';
		$paginator = new Paginator('mytisubmt', -1, $total, $default_sort_by, $columns);
		
		$sql = "select t.tracker_name, t.tracker_id, t.project_id, p.unix_name, p.project_name, ti.tracker_item_id, ti.summary, ti.priority, ti.details from tracker t, project p, tracker_item ti 
		where ti.submitted_by = '$user_id' 
		and ti.tracker_id = t.tracker_id 
		and t.project_id = p.project_id
		and ti.status_id=".Tracker::STATUS_OPEN . $paginator->getSQL(); //" order by t.project_id,t.tracker_id desc";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		
		$last_tracker = null;
		$last_project = null;
		
		/*$theader = array();
		$theader[] = Language::getSessText('Project.Project');
		$theader[] = Language::getSessText('Tracker.TabString');
		$theader[] = Language::getSessText('Common.Id');;
		$theader[] = Language::getSessText('TrackerItem.PriorityName');
		$theader[] = $paginator->insertLimitOptions(Language::getSessText('TrackerItem.SummaryName'));
		*/
		//$value .= $paginator->showLimitOptions();
		$sort = $paginator->getSortingOptions();
		$sort_by = $sort['sort_by'];
		$sort_order = $sort['order'];

		$theader = $paginator->getSortableTableHeaders($theme);
		$value .= $theme->tableTop($theader);		
		
		if ($rs->getRecordCount()>0) {
			$ttype1=array();
			$ttype2=array();
			while($rs->next()) {
				$ttype1[$rs->getInt('tracker_id')][] = '1';
				$ttype2[$rs->getInt('project_id')][] = '1';
			}
			$rs->beforeFirst();
			
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$tracker_name = $rs->getString('tracker_name');
				$project_name = $rs->getString('project_name');
				//if not using default sort, the grouping will get messed up
				if ($last_project != $rs->getString('project_id') || $default_sort_by != $sort_by) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id'))) . '" >' .  $project_name . '</a>';
					if ($default_sort_by != $sort_by) {
						$tmod[]='rowspan="1"';
					} else {
						$tmod[]='rowspan="'.count($ttype2[$rs->getInt('project_id')]).'" bgcolor="#FFFFFF"';
					}
				}				
				//if not using default sort, the grouping will get messed up
				if ($last_tracker != $rs->getString('tracker_id') || $default_sort_by != $sort_by) {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerItemBrowse',$this->getName(),'',array('tracker_id'=>$rs->getInt('tracker_id'))) . '" >' .  $tracker_name . '</a>';
					if ($default_sort_by != $sort_by) {
						$tmod[]='rowspan="1"';
					} else {
						$tmod[]='rowspan="'.count($ttype1[$rs->getInt('tracker_id')]).'" bgcolor="#FFFFFF"';
					}
				}
				$trow[] = $rs->getInt('tracker_item_id');
				$tmod[]='';
				$trow[] = $rs->getString('priority');
				$tmod[]='bgcolor="'.$theme->getPriorityColor($rs->getString('priority')).'"';
				$last_tracker = $rs->getString('tracker_id');
				$last_project = $rs->getString('project_id');

				// Text that will be used on the tooltip
				$tooltip = StringUtils::trimString($rs->getString('details'), 500);
				$tooltip = StringUtils::generateTooltip($tooltip);

				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'TrackerItemEdit',$this->getName(),'',array('tracker_id'=>$rs->getInt('tracker_id'),'tracker_item_id'=>$rs->getInt('tracker_item_id'))) . '" onmouseover="tooltip(\''.$tooltip.'\')" onmouseout="tooltipexit()">' .  $rs->getString('summary') . '</a>';
				$tmod[]='';
				$value .= $theme->tableRow($trow,$tmod);
			}
			//$value .= $theme->listPaginatorRow($paginator);
		} else {
			$value .= $theme->tableFullRow(Language::getSessText('My.NoSubmittedTracker'));
		}
		$value .= $theme->tableBottom();
		if(isset($paginator) && is_object($paginator)) {
			$value .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
		}

		$value .= '</div>';			
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;
	}

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function checkDownloadPerms($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$result = new PluginResult($this);
		$result->setResult(false); // false by default, looking for true
		// if it's for another section, exit
		if ($section!=TrackerItem::SECTION) {
			$result->setResult(false);
		} else {
			$ti = TrackerItemPeer::retrieveByPK($ref_id);
			if (is_object($ti)) {
				// first check if the project is public, if so, then give permission
				if ($ti->getTracker()->isPublic()) {
					$result->setResult(true);
				}
				$project = $ti->getTracker()->getProject();
				if (is_object($project)) {
					$perm = $project->getPermission();
					// check if the user has permissions for reading at least
					if ($perm->check($this->getName(),$ti->getTracker()->getPrimaryKey(),TrackerPlugin::READ)) {
						$result->setResult(true);
					}
				}
			}
		}
		return $result;
	}

	/**
	 *      return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params) {
		$sql="select q.project_id,sum(q.usage) as usage FROM (
		--tracker items
		SELECT t.project_id,sum(length(ti.details)*2) AS usage
			FROM tracker t JOIN tracker_item ti USING (tracker_id)
			GROUP BY project_id
		UNION
		--followups
		SELECT t.project_id,sum(length(tim.body)*2) AS usage
			FROM tracker t JOIN tracker_item ti USING (tracker_id) JOIN tracker_item_message tim USING (tracker_item_id)

			GROUP BY project_id
		UNION
		--attachments
		SELECT t.project_id,sum(fs.file_size+length(fsi.vectors)) AS usage
			FROM tracker t JOIN tracker_item ti USING (tracker_id),
			filesystem fs JOIN filesystem_idx fsi USING (filesystem_id)
			WHERE ti.tracker_item_id=fs.ref_id
			AND fs.section='trackeritem'
			GROUP BY project_id
		) q
		GROUP BY project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}


	/**
	 *
	 * return the 
	 *
	 * @param array $words_arr  the list of tags
	 * @param integer $project_id if null, then the search is a global search
	 * 
	 * @return ResultSet
	 */
	
	public function searchByTags(GFSearch $search){


		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/',',', $words);
		$tsearch_operators = array('(', ')', '!', '|','&');
		$replacements = array('','','','',',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if($wc == 0){
			return '';
		}
		for($i = 0; $i < $wc; $i++){
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		//Tracker
		if($project_id){
			$tracker_prj = " AND tracker. project_id=$project_id";
		} else {
			$tracker_prj = ' ';
		}

		//Get all the tracker ids involved
		$tracker_perms_sql = "SELECT DISTINCT tracker_item.tracker_id AS tracker_id 
		FROM tracker_item NATURAL JOIN tracker
		WHERE tracker_item.tracker_item_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='trackeritem'
		AND tag_name IN ($words) 
		GROUP BY tag_entry.ref_id
		HAVING COUNT(ref_id) =  $wc) $tracker_prj";
		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($tracker_perms_sql, ResultSet::FETCHMODE_NUM);
		
		$tracker_aux = '';
		//Get trackers and check permissions
		if($rs->getRecordCount() > 0){
			$trackers = array();
			while($rs->next()){
				$trackers[] = $rs->getInt(1);
			}
			$trs = TrackerPeer::retrieveByPks($trackers);
			
			//If user can read, add the tracker id to the tracker_aux sql sentence
			$trackers = array();
			foreach($trs as $tr){
				if($tr->getProject()->usesPlugin('tracker') && $tr->getShowTagsTab() && $tr->userCanRead()){
						$trackers[] = $tr->getTrackerId();
				}
			}
			if(count($trackers) == 0){
				// no trackers
				return null;
			} else {
				$tracker_aux = ' AND tracker_item.tracker_id IN (' . implode(',', $trackers) .')';
			}
		};

		$tracker_sql = "SELECT 'trackeritem' AS section,tracker_item.tracker_item_id AS ref_id,
		tracker_name AS tracker_name,
		tracker_id AS tracker_id,
		tracker_item.summary AS description, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name
		FROM tracker_item NATURAL JOIN tracker
		INNER JOIN project ON tracker.project_id = project.project_id
		WHERE tracker_item_id IN 
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='trackeritem'
		AND tag_name IN ($words) 
		GROUP BY tag_entry.ref_id
		HAVING COUNT(ref_id) =  $wc) $tracker_aux $tracker_prj";
		
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($tracker_sql, ResultSet::FETCHMODE_ASSOC);

		$ret = null;
		if($rs->getRecordCount() > 0){

			$Lang = Session::getSession()->getLanguage();
			while($rs->next()){
				$trow = array();

				if(!$project_id){
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">'. $rs->getString('prj_name') . '</a>';
				}
				$trow[] = $rs->getInt('ref_id');
				$trow[] = $Lang->getText('Tag.TrackerItemSection');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), 'TrackerItemBrowse','tracker','',array('tracker_id'=>$rs->getInt('tracker_id'))) . '">' .$rs->getString('tracker_name') .'</a>';
				$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), 'TrackerItemEdit', 'tracker','',array('tracker_item_id'=>$rs->getInt('ref_id'))) . '">'.$rs->getString('description')."</>";
				$ret .= $template->tableRow($trow);
			}

		}

		return $ret;

	}
		
	
	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	public function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}
		
		// Search within the project
		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			if ($search->getStartDate()) {
				$date_sql = " AND ti.open_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}
			if ($search->getSearchAttach()) {
				$attach_sql="UNION
				SELECT ti.tracker_item_id, rank(filesystem_idx.vectors, q) AS rank
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, tracker_item ti, tracker t
				WHERE filesystem_idx.vectors @@ q
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='trackeritem'
				AND fs.ref_id=ti.tracker_item_id
				AND ti.tracker_id=t.tracker_id
				$date_sql
				AND t.project_id='".$search->getProjectId()."' ";
			} else {
				$attach_sql='';
			}

			$sql="SELECT tracker_item.tracker_item_id,tracker_item.summary, 
			tracker_item.details, sum(s.rank) AS rank, tracker_item.tracker_id
			FROM tracker_item, (
				SELECT timi.tracker_item_id, rank(timi.vectors, q) AS rank
				FROM tracker_item_message_idx timi, to_tsquery('".$words."') q, tracker_item ti, tracker t
				WHERE timi.vectors @@ q
				AND timi.tracker_item_id=ti.tracker_item_id
				AND ti.tracker_id=t.tracker_id
				$date_sql
				AND t.project_id='".$search->getProjectId()."'
				$attach_sql
				UNION
				SELECT tii.tracker_item_id, rank(tii.vectors, q) AS rank
				FROM tracker_item_idx tii, to_tsquery('".$words."') q, tracker_item ti, tracker t
				WHERE tii.vectors @@ q
				AND tii.tracker_item_id=ti.tracker_item_id
				AND ti.tracker_id=t.tracker_id
				$date_sql
				AND t.project_id='".$search->getProjectId()."'
			) AS s
			WHERE s.tracker_item_id=tracker_item.tracker_item_id
			GROUP BY tracker_item.tracker_item_id,tracker_item.summary,tracker_item.details,tracker_item.tracker_id
			ORDER BY RANK DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();
			if ($rs->getRecordCount() > 0) {
				$ret = '';
				$current_tracker = '0'; // to see if the tracker has changed
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$ret = '<p><h3>'.Language::getSessText('Tracker.SearchResultsFor').'</h3></p>';
				$ret .= $template->tableTop(array('#',Language::getSessText('Tabs.Summary'),Language::getSessText('TrackerItemMessage.BodyName')));
				$i = 0;
				while ($rs->next()) {
					$tid = $rs->getInt('tracker_item_id');
					$trow = array();
					$trow[] = $tid;
					$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,'TrackerItemEdit',$this->getName(),'',array('tracker_id'=>$rs->getInt('tracker_id'),'tracker_item_id'=>$tid)) . '">' . $rs->getString('summary').'</a>';
					$trow[] = $rs->getString('details');
					$ret .= $template->tableRow($trow);
				}
				$ret .= $template->tableBottom();
				return $ret;
			} else {
				// zero results for search
				return '<p><h3>'.Language::getSessText('Tracker.SearchResultsFor').'</h3></p>' . $template->tableTop(array('#',Language::getSessText('Tabs.Summary'),Language::getSessText('TrackerItemMessage.BodyName'))) . $template->tableFullRow( Language::getSessText('Search.NoResults') ) . $template->tableBottom();
			}			
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
		

	}
	
	/**
	 *	Searches across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	private function globalSearch($search) {
		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			if ($search->getStartDate()) {
				$date_sql = " AND ti.open_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}
			if ($search->getSearchAttach()) {
				$attach_sql="UNION
				SELECT ti.tracker_item_id, rank(filesystem_idx.vectors, q) AS rank,
				headline(ti.summary, q) AS headline_summary, headline(ti.details, q) AS headline_details
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, tracker_item ti, tracker t
				WHERE filesystem_idx.vectors @@ q
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='trackeritem'
				AND fs.ref_id=ti.tracker_item_id
				AND ti.tracker_id=t.tracker_id
				$date_sql
				";
			} else {
				$attach_sql='';
			}

			$sql="SELECT 
					project.project_name, 		-- 1
					project.unix_name, 			-- 2
					sum(s.rank) AS rank,		-- 3
					s.headline_summary,			-- 4
					s.headline_details,			-- 5
					tracker_item.*				-- 6
				FROM tracker, project, tracker_item, (
					SELECT timi.tracker_item_id, rank(timi.vectors, q) AS rank,
					headline(ti.summary, q) AS headline_summary, headline(ti.details, q) AS headline_details
					FROM tracker_item_message_idx timi, to_tsquery('".$words."') q, tracker_item ti, tracker t
					WHERE timi.vectors @@ q
					AND timi.tracker_item_id=ti.tracker_item_id
					AND ti.tracker_id=t.tracker_id
					$date_sql
					$attach_sql
					UNION
					SELECT tii.tracker_item_id, rank(tii.vectors, q) AS rank,
					headline(ti.summary, q) AS headline_summary, headline(ti.details, q) AS headline_details
					FROM tracker_item_idx tii, to_tsquery('".$words."') q, tracker_item ti, tracker t
					WHERE tii.vectors @@ q
					AND tii.tracker_item_id=ti.tracker_item_id
					AND ti.tracker_id=t.tracker_id
					$date_sql
				) AS s
				WHERE s.tracker_item_id=tracker_item.tracker_item_id AND tracker_item.tracker_id=tracker.tracker_id
					AND tracker.project_id=project.project_id
				GROUP BY project.project_name,project.unix_name,tracker_item.tracker_item_id,tracker_item.tracker_id,
					tracker_item.status_id,tracker_item.priority,tracker_item.submitted_by,tracker_item.open_date,tracker_item.close_date,
					tracker_item.summary,tracker_item.details,tracker_item.last_modified_date,tracker_item.last_modified_by,
					tracker_item.sort_order,tracker_item.parent_id,s.headline_summary,s.headline_details
				ORDER BY RANK DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$template = $search->getTemplate();

			$count = 0;
			$ret = '';
			$current_tracker = '0'; // to see if the tracker has changed
			$ret = '<p><h3>'.Language::getSessText('Tracker.SearchResultsFor').'</h3></p>';
			$ret .= $template->tableTop(array(
						'#',
						Language::getSessText('Project.ListNameTitle'),
						Language::getSessText('Tabs.Summary'),
						Language::getSessText('TrackerItemMessage.BodyName')
					));
			while ($rs->next()) {
				$tracker_item = new TrackerItem();
				$tracker_item->hydrate($rs, 6);

				if (!$tracker_item->userCanRead()) {
					continue;
				}
				
				$count++;
				$project_unix_name = $rs->getString(2);
				$tid = $tracker_item->getTrackerItemId();
				$trow = array();
				$trow[] = $tid;
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(1).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,'TrackerItemEdit',$this->getName(),'',array('tracker_id'=>$tracker_item->getTrackerId(),'tracker_item_id'=>$tid)) . '">' . $rs->getString(4).'</a>';
				$trow[] = $rs->getString(5);
				$ret .= $template->tableRow($trow);
			}
			$ret .= $template->tableBottom();


			if ($count > 0) {
				return $ret;
			} else {
				// cero results for search
				return '<p><h3>'.Language::getSessText('Tracker.SearchResultsFor').'</h3></p>' . $template->tableTop(array('#',Language::getSessText('Tabs.Summary'),Language::getSessText('TrackerItemMessage.BodyName'))) . $template->tableFullRow( Language::getSessText('Search.NoResults') ) . $template->tableBottom();
			}			
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
		
		
	}
	
	/**
	 * returns the options to print in the combobox
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		$value = array();
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Tracker.ThisProjectTrackers');
		} else {
			$text = Language::getSessText('Tracker.AllProjectsTrackers');
		}
		$value[] = array(
			'value'	=> $this->getName(),
			'display'	=> $text
			);

		$result->setResult($value);
		return $result;		
	}

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return PluginResult
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}

	private function getSearchByTagsResult($params) {
		
		$result = new PluginResult($this);
		$search = new GFSearch($params);

		$result->setResult($this->searchByTags($search));

		return $result;

	}

	
	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$project = $params['project'];

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Tracker.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','tracker'),
				'id'=>$this->getName()
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Tracker',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/admin/',
				'description'=>Language::getSessText('Tracker.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	private function getProjectReportingNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
/*
		$tracker_links=array();
		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID,$params['project']->getProjectId());
		$trackers = TrackerPeer::doSelect($c);

		foreach($trackers as $tracker) {
			$tracker_links[]=array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackerQueryBrowse&amp;tracker_id='.$tracker->getTrackerId(),
				'description'=>$tracker->getTrackerName());
		}
		if (count($tracker_links) < 1) {
			return '';
		}
*/
		$elements=array(
		array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackersReport',
			'description'=>Language::getSessText('TrackerReporting.Title')),
		array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TimeTrackingReport',
			'description'=>Language::getSessText('TimeTrackingReporting.Title')));
		$result = new PluginResult($this);

		$arr = array(
			'title'=> Language::getSessText('Tracker.TabString'),
			'elements'=> $elements
		);
		$result->setResult($arr);
		return $result;
	}



	/**
	 *	getTrackerNavElements - get the local nav elements for the tracker
	 *	@return PluginResult
	 * /
	private function getTrackerNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Tracker Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function _buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

		$project = $params['project'];
		$theme = $params['theme'];
		
		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID,$project->getPrimaryKey());
		$c->addAscendingOrderByColumn(TrackerPeer::DATATYPE);
		$trackers = TrackerPeer::doSelect($c);
		$value=Language::getSessText('Tracker.TabString');
		foreach ($trackers as $tracker) {
			if ($tracker->userCanRead()) {
				if ($tracker->getDatatype()==Tracker::DATATYPE_TASK) {
					$icon='task_icon.gif';
				} else {
					$icon='tracker_icon.gif';
				}
				$value .= '<p><a href="' . GFUrl::getUrl()->projectURL($project,'TrackerItemBrowse','tracker','',array('tracker_id'=>$tracker->getPrimaryKey())).'"><img src="/images/'.$icon.'" height="16" width="16" border="0" alt="" />&nbsp;'. $tracker->getTrackerName() .'</a>
				( '. $tracker->getOpenCount() . ' ' . Language::getSessText('Tracker.OpenCount') . ' / ' . $tracker->getItemTotal() . ' ' . Language::getSessText('Tracker.TotalCount') .' )<br />'.
				$tracker->getDescription().'</p>';
			}
		}

		$result->setResult($value);
		return $result;
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->currentBaseUrl(); //projectURL($this->project,'','tracker');
	}
	
	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}

	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Tracker.NoAccess'); //descriptions
		$return[1][] = TrackerPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('Tracker.Read');
		$return[1][] = TrackerPlugin::READ;
		$return[0][] = Language::getSessText('Tracker.Tech');
		$return[1][] = TrackerPlugin::TECH;
		$return[0][] = Language::getSessText('Tracker.TechAdmin');
		$return[1][] = TrackerPlugin::TECHADMIN;
		$return[0][] = Language::getSessText('Tracker.Admin');
		$return[1][] = TrackerPlugin::ADMIN;
		
		return $return;
	}

	/**
	 * returns the possible site-wide admin role settings for this plugin
	 *
	 * @return array	the roles
	 */	
	function getAdminRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Tracker.None'); //descriptions
		$return[1][] = TrackerPlugin::NOACCESS;  // values
		$return[0][] = Language::getSessText('Tracker.Admin');
		$return[1][] = TrackerPlugin::ADMIN;
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings
	 *	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}		
		
		$roles = array();
		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID,$params['project']->getPrimaryKey());
		//add special case, project site-wide tracker admin
		$roles[0]['ref_id'] = 0;
		$roles[0]['section'] = Language::getSessText('Tracker.SiteWideAdmin');
		$roles[0]['description'] = Language::getSessText('Tracker.SiteWideAdmin.Description');
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = 'tracker';
		$roles[0]['settings'][] = $this->getAdminRoleSettings();
		//retrieve the trackers for the project
		$trackers = TrackerPeer::doSelect($c);
		foreach ($trackers as $tracker) {
			$key = $tracker->getPrimaryKey();
			$roles[$key]['ref_id'] = $key;
			$roles[$key]['section'] = Language::getSessText('Tracker.Roles.Section'); // the displayed text in "section" field
			$roles[$key]['description'] = Language::getSessText('Tracker.TrackerWideAdmin.Description');
			$roles[$key]['subsection'] = $tracker->getTrackerName(); // the displayed text in the "subsection" field
			$roles[$key]['settings'][0] = $this->getName();// the section for the DB, one for all the settings
			$roles[$key]['settings'][] = $this->getAvailableRoleSettings();
		}
		
		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}
	
	/**
	 * returns the settings for the tracker posting
	 *
	 * @return array
	 */
	function getPostingObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Tracker.NoAnon'); //descriptions
		$return[1][] = TrackerPlugin::NOANON; // values
		$return[0][] = Language::getSessText('Tracker.Anon');
		$return[1][] = TrackerPlugin::ANON;
		return $return;		
	}

	/**
	 * returns the settings for the tracker status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Tracker.Private'); //descriptions
		$return[1][] = TrackerPlugin::ISPRIVATE; // values
		$return[0][] = Language::getSessText('Tracker.Public');
		$return[1][] = TrackerPlugin::ISPUBLIC;
		return $return;		
	}	
	
	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$choices = array();
		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID,$params['project']->getPrimaryKey());
		//retrieve the tracker items for the project
		$trackers = TrackerPeer::doSelect($c);
		$i = 0;
		foreach ($trackers as $tracker) {
			$key = $tracker->getPrimaryKey();
			$choices[$i]['ref_id'] = $key;
			$choices[$i]['section'] = Language::getSessText('Tracker.Privileges');
			$choices[$i]['subsection'] = $tracker->getTrackerName();
			$choices[$i]['description'] = Language::getSessText('Tracker.TrackerWideAdmin.Description');
			$choices[$i]['settings'][0] = 'trackerpublic';
			$choices[$i]['settings'][] = $this->getStatusObserverSettings();
			$i++;
		}

		$result = new PluginResult($this);
		$result->setResult($choices);
		return $result;
	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */
	function updateObserverSettings($params) {
		$project = $params['project'];
		$data = $params['data'];

		$onedata = current($data);
		while ($onedata != null) {
			$section = key($data);
			switch ($section) {
				case 'trackerpublic' : {
					$oneitem = current($onedata);
					while ($oneitem != null) {
						$ref_id = key($onedata);
						$value = $oneitem;
						//retrieve the tracker and update it
						$c = new Criteria();
						$c->add(TrackerPeer::TRACKER_ID,$ref_id);
						try {
							$tracker = TrackerPeer::doSelectOne($c);
						} catch (Exception $e) {
							// something wrong happened
							echo $e->getMessage();
							next($onedata);
							$oneitem = current($onedata);
							continue;	
						}
						$tracker->setIsPublic($value);
						$tracker->save();
						next($onedata);
						$oneitem = current($onedata);					
					}		
					break;
				}
				default : { 
					//skip it
					continue;
				}
			}
			next($data);
			$onedata = current($data);
		}		
	}
	
	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id,$section) {
		switch ($section) {
			case 'trackerpublic' : {
				$c = new Criteria();
				$c->add(TrackerPeer::TRACKER_ID,$ref_id);
				try {
					$tracker = TrackerPeer::doSelectOne($c);
				} catch (Exception $e) {
					echo $e->getMessage();
					return '';
				}
				return $tracker->getIsPublic();
				break;
			}
			default : {
				return '';
			}
		}
	}	
	
	private function scmCommit($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$scmcommit = $params['commit'];
		$log = $scmcommit->getMessageLog();
		
		// $params['associations'] has all the associations, including tracker
		// items
		
		foreach ($params['associations'] as $association) {
			list($section, $ref_id) = $association;
			if ($section != TrackerItem::SECTION) {
				continue;
			}
			
			$tracker_item_id = $ref_id;
			// see if the item belongs to the project
			$item = TrackerItemPeer::retrieveByPk($tracker_item_id);
			if (is_null($item)) continue;
			if ($item->getTracker()->getProjectId() != $project->getProjectId()) continue;
			
			// Item belongs to the project. Add entry to tracker_item_scm_commit.
			// Make sure the entry doesn't exist first
			$c = new Criteria();
			$c->add(TrackerItemScmCommitPeer::SCM_COMMIT_ID, $scmcommit->getScmCommitId());
			$c->add(TrackerItemScmCommitPeer::TRACKER_ITEM_ID, $tracker_item_id);
			$trackeritem_commit = TrackerItemScmCommitPeer::doSelectOne($c);
			if (is_null($trackeritem_commit)) {
				$trackeritem_commit = new TrackerItemScmCommit();
				$trackeritem_commit->setScmCommitId($scmcommit->getScmCommitId());
				$trackeritem_commit->setTrackerItemId($tracker_item_id);
				$trackeritem_commit->save();
			}
			
			// Now fire the tracker commit trigger
			try {
				$item->commitTrigger($trackeritem_commit);
			} catch (Exception $ex) {
				// Usually exceptions are thrown if an invalid transition is made.
				// In this case, die silently because there's not much we can do
			}	
		}
		
		return null;
	}

	/**
	 *	return resultset of an array to populate the ID search select box
	 *	@return PluginResult
	 */
	function searchByIDList() {
		$pr=new PluginResult($this);
		$pr->setResult(array(TrackerItem::SECTION => 'Tracker Item'));
		return $pr;
	}
	/**
	 *	return result set of URL to redirect to
	 *	@param $params array
	 *	@return PluginResult
	 */
	function searchByIDAction($params) {
		if (isset($params['searchtype']) && $params['searchtype'] == TrackerItem::SECTION) {
			$pr=new PluginResult($this);
			$ti=TrackerItemPeer::retrieveByPk($params['searchid']);
			if (!is_object($ti)) {

			} else {
				$pr->setResult($ti->getTrackerItemUrl());
			}
			return $pr;
		}
	}
	/**
	 * It returns a list of involved tracker items.
	 * A tracker item is identified if [#(NUMBER)] if found.
	 *
	 * @param   string   $Log Log message to be parsed.
	 *
	 * @return  boot    Returns true if check passed.
	 */
	private function getInvolvedTrackerItems($Log) {
		preg_match_all('/[[]#[\d]+[]]/', $Log,  $Matches );
		foreach($Matches as $Match) {
			$Result = preg_replace ('/[[]#([\d]+)[]]/', '\1', $Match);
		}
		return $Result;
	}	
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);
		
		$c = new Criteria();
		$c->addJoin(RoleSettingPeer::ROLE_ID, RolePeer::ROLE_ID);
		$c->add(RolePeer::PROJECT_ID, $project->getProjectId());
		$rss = RoleSettingPeer::doSelect($c);

		$crit = new Criteria();
		$crit->add(TrackerPeer::PROJECT_ID, $template_project->getProjectId());
		$trackers = TrackerPeer::doSelect($crit);
		
		foreach($trackers as $tracker) {
			$new_tracker = new Tracker();
			$new_tracker->setTrackerName($tracker->getTrackerName());
			$new_tracker->setProject($project);
			$new_tracker->setDescription($tracker->getDescription());
			$new_tracker->setIsPublic($tracker->getIsPublic());
			$new_tracker->setRestrictBrowse($tracker->getRestrictBrowse());
			$new_tracker->setEmailAllUpdates($tracker->getEmailAllUpdates());
			$new_tracker->setEmailAddress($tracker->getEmailAddress());
			$new_tracker->setDuePeriod($tracker->getDuePeriod());
			$new_tracker->setSubmitInstructions($tracker->getSubmitInstructions());
			$new_tracker->setBrowseInstructions($tracker->getBrowseInstructions());
			$new_tracker->setDatatype($tracker->getDatatype());			
			$new_tracker->cloneFields($tracker);
			$new_tracker->setShowDependenciesTab($tracker->getShowDependenciesTab());
			$new_tracker->setShowDuplicatesTab($tracker->getShowDuplicatesTab());
			$new_tracker->setShowAssociationsTab($tracker->getShowAssociationsTab());
			$new_tracker->setShowTimeTrackingTab($tracker->getShowTimeTrackingTab());
			$new_tracker->setShowTagsTab($tracker->getShowTagsTab());
			$new_tracker->save();

			// search the role_settings and let's see if for this item we have one set
			foreach ($params['role_settings'] as $role_setting) {
				if (strstr($role_setting['section'],$this->getName()) && $role_setting['ref_id']==$tracker->getPrimaryKey()) {
					$role_id = $role_setting['role_id'];
					$section = $role_setting['section'];
					$ref_id = $new_tracker->getPrimaryKey();
					$value = $role_setting['value'];
					$stmt = $con->createStatement();
					
					// Note: when $new_tracker->save() is called, it creates default entries on the role_setting
					// table. We must override them.
					$sql = "DELETE FROM role_setting WHERE role_id='$role_id' AND section='$section' AND ref_id='$ref_id'";
					$rs = $stmt->executeQuery($sql);

					$sql = "insert into role_setting (role_id,section,ref_id,value) values ('$role_id','$section','$ref_id','$value')";
					$rs = $stmt->executeQuery($sql);
				}
			}

		}
	}
	
	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$lang = Session::getSession()->getLanguage();

		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != 'tracker' && $section != 'trackeritem' && $section != self::PENDING_TASKS_SECTION) return;		// only deal with package monitor
		$pr = new PluginResult($this);
		
		// Pending tasks email cronjob -- doesn't need to check permissions
		if ($section == self::PENDING_TASKS_SECTION) {
			$pr->setResult(true);
			return $pr;
		}
		
		if ($section == 'tracker') {
			$obj = TrackerPeer::retrieveByPk($ref_id);
		} elseif ($section == 'trackeritem') {
			$obj = TrackerItemPeer::retrieveByPk($ref_id);
		}

		if (is_null($obj) || !$obj->userCanRead()) {
			$pr->setResult(false);
		} else {
			$pr->setResult(true);
		}

		return $pr;
	}

	/**
	 * Process an incoming email
	 */
	private function processMailGateway($params) {
		if ($params['section'] == Tracker::SECTION) {
			$this->gatewayTrackerItem($params);
		} else if ($params['section'] != TrackerItem::SECTION) return;
		
		$tracker_item_id = $params['ref_id'];
		$user = $params['from_user'];
		$subject = $params['subject'];
		$body = $params['body'];
		
		$text = StringUtils::getTextBetweenMarkers($body, TrackerItemNotification::TRACKER_MAIL_MARKER, TrackerItemNotification::TRACKER_MAIL_MARKER);
		// Last line in $message may be "> " because the user hit "reply". Remove that.
		$text = preg_replace('/> ?$/m', '', $text);
		
		if (empty($text)) return;		// No message to post
		
		$item = TrackerItemPeer::retrieveByPk($tracker_item_id);
		if (is_null($item)) {
			throw new Exception(Language::getSessText('TrackerItem.InvalidId', array($tracker_item_id)));
		}
		
		$new_message = new TrackerItemMessage();
		$new_message->setTrackerItem($item);
		$new_message->setSubmittedBy($user->getUserId());
		$new_message->setBody(Validator::sanitizeString($text));
		$new_message->save();
		
		// Notify the changes to users that are monitoring the item
		$notification = new TrackerItemNotification($item);
		if ($item->getTracker()->getEmailAllUpdates()) {
			// This sends notification to the e-mail addresses defined in
			// "email_address" field
			$notification->notifyUpdate();
		}
		NotificationManager::getManager()->queue($notification);
		
		return;		
	}
	
	private function gatewayTrackerItem($params) {

		$tracker_id = $params['ref_id'];
		$submitter = $params['from_user'];
		$summary = $params['subject'];
		$body = trim($params['body']);

		$Tracker = TrackerPeer::retrieveByPk($tracker_id);
		if (is_null($Tracker)) {
			throw new Exception(Language::getSessText('Tracker.InvalidId'), array($tracker_id));
		}

		if (empty($body)) return;		// No description for the item
		
		$lines = explode("\n", $body);
		$options = array();
		$details = '';
		
		// Get the "options" first
		for($i=0;$i<count($lines);$i++) {
			// options should be on the top of the email
			if (strpos($lines[$i], ':') !== false) {
				list($option, $value) = explode(':', trim($lines[$i]), 2);
				$options[strtolower(trim($option))] = trim($value);
			} else {
				// no more options. exit loop.
				break;
			}
		}
		// now get the actual details for the item
		for ($j=$i;$j<count($lines);$j++) {
			$details .= trim($lines[$j])."\n";
		}


		$TrackerItem = new TrackerItem();
		$Tracker->addTrackerItem($TrackerItem);
		
		// handle and add "options"
		$assignees = array();
		foreach ($options as $option => $value) {
			switch ($option) {
				case 'priority': {
					$TrackerItem->setPriority(Validator::sanitizeInt($value));
					break;
				}
				case 'assignees': {
					if ($Tracker->userIsAdmin()) {
						$users = explode(',', $value);
						foreach ($users as $user) {
							if (is_numeric($user)) {
								$assignees[] = trim($user);
							} else {
								$assignee = UserPeer::retrieveByUnixName(trim($user));
								if (is_object($assignee)) {
									$assignees[] = $assignee->getUserId();
								}
							}
						}
					}
					break;
				}
				case 'start date': {
					if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
						$TrackerItem->setOpenDate(Validator::sanitizeDate($value));
					}
				}
				case 'end date': {
					if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
						$TrackerItem->setCloseDate(Validator::sanitizeDate($value));
					}
				}
			}
		}
		// Add assignees
		$assignees = (count($assignees) > 0) ? $assignees : array(100); // make sure at least Nobody is set
		$TrackerItem->addAssignees($assignees);
		
		// Now handle extra fields, use default values
		$criteria = new Criteria();
		$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
		$defaultValues = $TrackerItem->getTracker()->getTrackerDefaultSelected($criteria);
		$extraFieldIds = $TrackerItem->getTracker()->getTrackerExtraFields();

		$extraFields = array();
		
		foreach ($extraFieldIds as $field) {
			$efid = $field->getTrackerExtraFieldId();
			if ($field->getFieldType()==TrackerExtraField::TYPE_MULTISELECT 
				|| $field->getFieldType()==TrackerExtraField::TYPE_CHECKBOX) {
				$extraFields[$efid] = isset($defaultValues[$efid]) ? $defaultValues[$efid] : array(100);
			} else if ($field->getFieldType()==TrackerExtraField::TYPE_SELECT 
				|| $field->getFieldType()==TrackerExtraField::TYPE_RADIO
				|| $field->getFieldType()==TrackerExtraField::TYPE_STATUS
				|| $field->getFieldType()==TrackerExtraField::TYPE_RELEASE) {
				$extraFields[$efid] = isset($defaultValues[$efid]) ? $defaultValues[$efid][0] : 100;
			} else {
				$extraFields[$efid] = '';
			}
		}
		if (count($extraFields) > 0) {
			$TrackerItem->addExtraFields($extraFields);
		}
	
		$TrackerItem->setSubmittedBy($submitter->getUserId());
		$TrackerItem->setSummary(Validator::sanitizeString($summary));
		$TrackerItem->setDetails(Validator::sanitizeString($details));
		$TrackerItem->save();
		
		return;
	}

	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$section = self::SECTION;

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];
		
		if (!isset($data[$section])) {
			return;
		}
		
		foreach ($data[$section] as $ref_id => $value) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
			$c->add(RoleSettingPeer::SECTION, $section);
			$c->add(RoleSettingPeer::REF_ID, $ref_id);
			$rolesetting = RoleSettingPeer::doSelectOne($c);
			if (!is_object($rolesetting)) {
				// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
				$rolesetting = new RoleSetting();
				$rolesetting->setSection($section);
				$rolesetting->setRefId($ref_id);
				$rolesetting->setRoleId($role->getPrimaryKey());
			}
			
			$rolesetting->setValue($value);
			$rolesetting->save();						
		}	
		
		return;
	}

	private function buildRssFeed($params) {
		$section = $params['section'];
		$tracker = TrackerPeer::retrieveByPk($params['ref_id']);
		$project = $params['project'];
		$url = GFUrl::getURL();

		$result = new PluginResult($this);
		
		if (!is_object($tracker) || !$tracker->userCanRead()) {
			$result->setResult(false);
			return $result;
		}
		
		$c = new Criteria();
		$c->add(TrackerItemPeer::TRACKER_ID, $params['ref_id']);
		$c->add(TrackerItemPeer::STATUS_ID, 1);
		$c->addDescendingOrderByColumn(TrackerItemPeer::OPEN_DATE);
		$c->setLimit(20);
		$items = TrackerItemPeer::doSelect($c);
		
		$channelName = 'Gforge Advanced Server - '.$project->getName().' - '.$tracker->getTrackerName();
		
		$rssarr = array();
		$rssarr['channel']['name'] = $channelName;
		$rssarr['channel']['url'] = $url->getFullURL($url->baseURL());
		$rssarr['channel']['description'] = 'Gforge Advanced Server RSS';
		
		$i = 0;
		foreach ($items as $item) {
			$rssarr['items'][$i]['channel_name'] = $channelName;
			$rssarr['items'][$i]['title'] = $item->getSummary();
			$rssarr['items'][$i]['link'] = '<![CDATA['.$url->getFullURL($url->projectURL($project, 'TrackerItemEdit', $section, '', array('tracker_item_id' => $item->getTrackerItemId()))).']]>';
			$rssarr['items'][$i]['author'] = UserPeer::retrieveByPk($item->getSubmittedBy())->getName().' &lt;'.UserPeer::retrieveByPk($item->getSubmittedBy())->getEmail().'&gt;';
			$rssarr['items'][$i]['guid'] = '<![CDATA['.$url->getFullURL($url->projectURL($project, 'TrackerItemEdit', $section, '', array('tracker_item_id' => $item->getTrackerItemId()))).']]>';
			$rssarr['items'][$i]['description'] = $item->getDetails();

			$i++;
		}
		
		if (array_key_exists('channel', $rssarr) && array_key_exists('items', $rssarr)) {
			$result->setResult($rssarr);
		} else {
			$result->setResult(false);
		}
		return $result;
	}
	
	private function getProjectAuditTrail($params) {
		$project = $params['project'];
		$audit_rs = $params['audit_trail'];		// This is a recordset
		
		$section = $audit_rs->getString('section');
		$field_name = $audit_rs->getString('field_name');
		$new_value = $audit_rs->getString('new_value');
		
		if ($section != Project::SECTION) {
			// Not the project-level audit trail
			return;
		}
		
		switch ($field_name) {
		case TrackerItem::SECTION:
			if ($new_value == '_deleted_') {
				// Deleted tracker item. The old value is the summary
				// of the tracker item
				$summary = $audit_rs->getString('old_value');
				$desc = Language::getSessText('TrackerItem.ProjectAudit.Deleted', array($summary));
			} else {
				return;
			}
			break;
		case Tracker::SECTION:
			if ($new_value == '_deleted_') {
				// Deleted tracker. The old value is the tracker name
				$tracker_name = $audit_rs->getString('old_value');
				$desc = Language::getSessText('Tracker.ProjectAudit.Deleted', array($tracker_name));
			} else {
				return;
			}
			break;
		default:
			return;
		}

		$result = new PluginResult($this);
		$result->setResult($desc);
		return $result;
	}

	function getProjectSettingsForm($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) {
			return;
		}
		
		$Lang = Session::getSession()->getLanguage();
		$theme = Session::getSession()->getTheme();
		
		$crit = new Criteria();
		$crit->add(TrackerPeer::PROJECT_ID, $project->getProjectId());
		$crit->addAscendingOrderByColumn(TrackerPeer::DATATYPE);
		$arr = TrackerPeer::doSelect($crit);
		
		$form = array();
		$form['title'] = Language::getSessText('Tracker.ProjectSettingsTitle');
		
		$contents = '';

		$theader = array();
		$theader[] = $Lang->getText('Tracker.DatatypeName');
		$theader[] = $Lang->getText('Tracker.TrackerNameName');
		$theader[] = $Lang->getText('Tracker.ProjectSettingsSettings');
		$contents .= $theme->tableTop($theader);
		
		$url = GFUrl::getUrl();
		
		if (count($arr) < 1) {
			$contents .= $theme->tableFullRow($Lang->getText('Tracker.NoneFound'));
		} else {
			reset($arr);
			$ttype=array();
			foreach ($arr as $item) {
				$ttype[$item->getDataType()][]=$item;
			}

			reset($arr);
			$lasttype='-1';

			foreach ($arr as $item) {
				$trow = array();
				$tmod = array();
				if ($item->getDataType() != $lasttype) {
					$lasttype=$item->getDataType();
					$tmod[]='rowspan="'.count($ttype[$item->getDataType()]).'" bgcolor="#FFFFFF"';
					$trow[]=(($item->getDataType()==Tracker::DATATYPE_TASK) ? 'Tasks' : 'Issues');
				}
				$trow[]=$item->getTrackerName();
				$tmod[]='';
				$trow[]='<a href="'. $url->projectUrl($project, 'TrackerEdit', $this->getName(), 'admin', array('tracker_id' => $item->getTrackerID(), 'redirect_to' => $url->currentUrl())).'">'. $Lang->getText('Tracker.ProjectSettingsSettings') .'</a>';
				$tmod[]='';

				$contents .= $theme->tableRow($trow,$tmod, false);
			}

		}
		$contents .= $theme->tableBottom();
		
		$form['contents'] = $contents;

		$result = new PluginResult($this);
		$result->setResult($form);
		return $result;
	}
	
	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	/*private function getLocalNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Tracker',
			'id'=>$this->getName(),
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/',
				'description'=>Language::getSessText('Tracker.TrackerHome'))
			)
		);
		
		$tracker_links=array();
		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID,$params['project']->getProjectId());
		$trackers = TrackerPeer::doSelect($c);

		foreach($trackers as $tracker) {
			$tracker_links[]=array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackerItemBrowse&amp;tracker_id='.$tracker->getTrackerId(),
				'description'=>$tracker->getTrackerName());
		}
		if (count($tracker_links) > 0) {
			$arr['elements'] = array_merge($arr['elements'], $tracker_links);
		}
		
		$result->setResult($arr);
		return $result;
		
	}*/

}

/*
array(
		['title']='Title';
		['elements']= array(
		array('link'=>'/path/to/file/?action',
			'description'='Title'),
		array('link'=>'/path/to/file/?action',
			'description'='Title'),
		array('link'=>'/path/to/file/?action',
			'description'='Title')
		)
*/

