<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a TrackerItem and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('core/FilesystemPeer.php');
require_once('plugins/tracker/lib/TrackerItemTime.php');
require_once('plugins/tracker/lib/TrackerItemRelation.php');
require_once('plugins/tracker/lib/TrackerItemMessage.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

//force login
$sess = Session::getSession();
if (!$sess->isLoggedIn()) {
	throw new NotLoggedInException();
}
$Project = GFUrl::getUrl()->getProject();

$Tracker=TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!$Tracker || !is_object($Tracker)) {
	throw new Exception('No Such Tracker');
}
$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
$con->begin();

//no items selected - redirect to TrackerItemBrowse
if (!isset($_REQUEST['tid'])) {
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $language->getText('TrackerMassUpdate.NoItemsSelected'),'tracker_id'=>$Tracker->getTrackerId())));
}

foreach ($_REQUEST['tid'] as $tracker_item_id) {
	$TrackerItem = TrackerItemPeer::retrieveByPk($tracker_item_id);
	if (!$TrackerItem || !is_object($TrackerItem)) {
		throw new Exception('No Such Item');
	}
	//validate that this is a legitimate trackeritem for this tracker and user can add items
	if (!$TrackerItem || !is_object($TrackerItem) 
		|| $TrackerItem->getTracker()->getTrackerId() != $Tracker->getTrackerId() 
		|| !$TrackerItem->userCanUpdate()) {
		throw new PermissionDeniedException();
	}

	try {
		$update=false;
		if (isset($_REQUEST['priority']) && !empty($_REQUEST['priority'])) {
			$TrackerItem->setPriority( Validator::sanitizeInt($_REQUEST['priority']) );
			$update=true;
			
		}
		if (isset($_REQUEST['assigned_to']) && !empty($_REQUEST['assigned_to'])) {
			$TrackerItem->updateAssignees(array($_REQUEST['assigned_to']));
			$update=true;
		}
		if (array_key_exists('extra_field',$_POST)) {
			$updateef=false;
			$keys=array_keys($_REQUEST['extra_field']);
			$efd = $TrackerItem->getArrangedExtraFieldData();
			foreach ($keys as $key) {
				$data=$_REQUEST['extra_field'][$key];
				if (isset($data) && !empty($data)) {
					$ef=TrackerExtraFieldPeer::retrieveByPk($key);
					if ($ef && is_object($ef)) {
						if ($ef->getFieldType()==TrackerExtraField::TYPE_MULTISELECT 
						   || $ef->getFieldType()==TrackerExtraField::TYPE_MEMBERS
							|| $ef->getFieldType()==TrackerExtraField::TYPE_CHECKBOX) {
							$efd[$key]=array($data);
						} else {
							$efd[$key]=$data;
						}
					}
					$updateef=true;
				}
			}
			if ($updateef) {
				$update=true;
				$TrackerItem->updateExtraFields($efd);
			}
		}
		if ($update) {
			$TrackerItem->setLastModifiedDate(time());
			$TrackerItem->setLastModifiedBy($sess->getUser()->getUserId());
			$TrackerItem->save();
		}

		$language = Language::getSessLang();
	} catch (InvalidDataException $ex) {
		$con->rollback();
		$feedback = implode('<br />', $ex->getErrors());
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $feedback,'tracker_id'=>$Tracker->getTrackerId())));
	}
}
$con->commit();
GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $language->getText('TrackerItem.EditFeedback'),'tracker_id'=>$Tracker->getTrackerId())));

?>
