<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('exceptions/MissingDataException.php');
require_once('exceptions/PermissionDeniedException.php');

if (!Session::getSession()->isLoggedIn()) {
	throw new PermissionDeniedException();
}

$project = GFUrl::getUrl()->getProject();
$lang = Session::getSession()->getLanguage();
$user = Session::getSession()->getUser();

require_once('plugins/tracker/lib/html/TrackerTemplate.php');
$theme = new TrackerTemplate();

if (!isset($_GET['tracker_item_id'])) {
	throw new MissingDataException('Missing tracker item ID');
}

$tracker_item = TrackerItemPeer::retrieveByPk($_GET['tracker_item_id']);
if (is_null($tracker_item)) {
	throw new PermissionDeniedException();
}
$tracker = $tracker_item->getTracker();
if (!$tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

$to_tracker_id = (isset($_POST['to_tracker_id'])) ? $_POST['to_tracker_id'] : 0;

$params = array(
		'project' => $project,
		'tracker' => $tracker,
		'tracker_item' => $tracker_item,
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'action' => isset($action) ? $action : '',
		'title' => $lang->getText('TrackerItem.MoveItemTitle')
		);
$theme->header($params);

echo '<p>'.$lang->getText('TrackerItem.MoveItemDescription', array('[#'.$tracker_item->getTrackerItemId().'] '.$tracker_item->getSummary())).'</p>';

echo '<form method="post" action="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemMoveAction', array('tracker_item_id' => $tracker_item->getTrackerItemId())).'">';


$c = new Criteria();
$c->add(PluginPeer::PLUGIN_NAME, 'tracker');
$tracker_plugin = PluginPeer::doSelectOne($c);

if (is_object($tracker_plugin)) {
	$tracker_plugin_id = $tracker_plugin->getPluginId();
} else {
	$tracker_plugin_id = 0;
}

$user_projects = $user->getAllProjects();
foreach ($user_projects as $project_to) {
	if ($project_to->getStatus() != Project::STATUS_ACTIVE) {
		// show only active projects
		continue;
	}
	
	//Get project-plugin object
	$c = new Criteria();
	$c->add(ProjectPluginPeer::PROJECT_ID, $project_to->getProjectId());
	$c->add(ProjectPluginPeer::PLUGIN_ID, $tracker_plugin_id);
	$project_plugin = ProjectPluginPeer::doSelectOne($c);

	//Check if plugin is enabled
	if (is_object($project_plugin)) {
		// Get the trackers for this project
		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID, $project_to->getProjectId());
		$c->addAscendingOrderByColumn(TrackerPeer::TRACKER_NAME);
		$trackers = TrackerPeer::doSelect($c);
		foreach ($trackers as $tracker_to) {
			if ($tracker->getTrackerId() == $tracker_to->getTrackerId()) {
				// don't allow copying to the same tracker
				continue;
			}
	
			if ($tracker_to->userCanRead()) {
				echo '<input type="radio" name="to_tracker_id" value="'.$tracker_to->getTrackerId().'" ';
				if ($tracker_to->getTrackerId() == $to_tracker_id) echo 'checked="checked"';
				echo '/> <strong>'.$project_to->getProjectName().'</strong> :: '.$tracker_to->getTrackerName().'<br />' ;
			}
		}
	}
}
echo '<br />';
echo '<span class="warning">'.$lang->getText('TrackerItem.MoveItemWarning').'</span>';
echo '<br />';
echo '<br />';
echo HTMLWidget::button("submit",$lang->getText('TrackerItem.MoveItemButton'),"submit","","");
echo '</form>';

$theme->footer($params);

?>
