<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel A. Perez	daniel@gforgegroup.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/html/ProjectTemplate.php');
require_once('lib/plugin/PluginResult.php');

class WikiTemplate extends ProjectTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='wiki';
		$Lang = Session::getSession()->getLanguage();
		if (isset($params['title'])) {
			$params['title']=' '.$Lang->getText('Wiki.TabString').' > '.$params['title'];
		} else {
			Throw new exception('Missing title in wiki template');
		}

		//do a permission check
		$wps = WikiProjectSettingPeer::retrieveByPk($params['project']->getProjectId());
		if (!is_object($wps) || !$wps->getIsPublic()) {
			$perm = $params['project']->getPermission();
			if (!$perm->check(WikiPlugin::SECTION, 0, WikiPlugin::READ)) {
				Throw new PermissionDeniedException();
			}
		}
		
		if (isset($params['wiki'])) {
			$links = $params['wiki']->getBreadCrumbs();
			foreach ($links as $link) {
				$this->addBreadCrumb($link);
			}
		}
		$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','wiki').'">'.$Lang->getText('Wiki.TabString').'</a>');		

		return parent::header($params);
	}

	/**
	 *	getPluginNavElements - this overrides parent class and adds its own 
	 *	plugin listener
	 *	@return	PluginResultSet
	 */
	function getPluginNavElements($params) {

	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];
		
		
		$elements = array();

		//list all wiki pages
		$elements[] = array('link' => $url->projectURL($project,'','wiki','', array('action'=>'WikiPageBrowse')),
			'description' => $lang->getText('WikiPage.Browse'));
		// Monitoring
		if (!is_null($user)) {
			// Whole Wiki monitoring
			if (WikiPlugin::userIsMonitoring($user, $project)) {
				$elements[] = array('link' => $url->getMonitorUrl(WikiPlugin::SECTION, $project->getProjectId(), $url->currentUrl(), false),
					'description' => $lang->getText('Wiki.StopMonitoring'), true);
			} else {
				$elements[] = array('link' => $url->getMonitorUrl(WikiPlugin::SECTION, $project->getProjectId(), $url->currentUrl(), true),
					'description' => $lang->getText('Wiki.Monitor'), true);
			}
			
			$elements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>WikiPlugin::SECTION,'ref_id'=>$project->getProjectId(),'description'=>'Wiki','type'=>'plugin','return_to'=>$_SERVER['REQUEST_URI'])),
				'description' => $lang->getText('Monitor.UsersMonitoringPlugin'));

			// Wiki page monitoring
			if (isset($params['page']) && is_object($params['page'])) {
				$page = $params['page'];
				if ($page->userIsMonitoring($user)) {
					$elements[] = array('link' => $url->getMonitorUrl(WikiPage::SECTION, $page->getPrimaryKey(), $url->currentUrl(), false),
						'description' => $lang->getText('WikiPage.StopMonitoring'), true);
				} else {
					$elements[] = array('link' => $url->getMonitorUrl(WikiPage::SECTION, $page->getPrimaryKey(), $url->currentUrl(), true),
						'description' => $lang->getText('WikiPage.Monitor'), true);
				}
			}
		}
		//wiki-page specific links
		if (isset($params['page']) && is_object($params['page'])) {
				$page = $params['page'];
				$elements[] = array('link' => $url->projectURL($project,'','wiki','', array('action'=>'PageInfo', 'info'=>urlencode($page->getPageName()) )), 'description' => $lang->getText('Wiki.Info'));
				$elements[] = array('link' => $url->projectURL($project,'','wiki','', array('action'=>'PageRefs', 'id'=>$page->getPrimaryKey() )), 'description' => $lang->getText('Wiki.Backlinks'));

				if ($loggedin) {
					$elements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>WikiPage::SECTION,'ref_id'=>$page->getPrimaryKey(),'description'=>$page->getPagename(),'type'=>'item','return_to'=>$_SERVER['REQUEST_URI'])),
						'description' => $lang->getText('Monitor.UsersMonitoringItem'));		
				}
		}
/*	
		if ($loggedin) {
			if ($project->getPermission()->isAdmin()) {
				$elements[] = array('link' => $url->projectURL($project,'','wiki','admin', ''),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}
*/
		if (count($elements) > 0) {
			$ret[] = array(
				'title' => $lang->getText('Wiki.TabString'),
				'elements'=> $elements
			);
		}
		if (isset($ret)) {
			return $ret;
		}
	}
	
	function generateRSSLinks() {
		$out = array();
		$url = GFUrl::getUrl();
		$project = $url->getProject();
		$project_id = $project->getProjectId();
	
//		if (is_object($project) && $project->isPublic()) {
			$out[] = array('title' => GFConf::get('system.name').' - '.$project->getName().' - Wiki - Latest pages',
							'url' => $url->getFullURL($url->baseURL('Rss', '', '', array('section' => 'projectwiki', 'project_id' => $project_id)))
						);
			$out[] = array('title' => GFConf::get('system.name').' - '.$project->getName().' - Wiki - Latest messages',
							'url' => $url->getFullURL($url->baseURL('Rss', '', '', array('section' => 'projectwikimessages', 'project_id' => $project_id)))
						);
//		}

		return $out;
	}
	function getExtraCSS() {
		$css = array();
/*		
		$css[] = '/themes/'. $this->getName() .'/textwiki.css';
		$css[] = 'textwiki.css';
*/
		return $css;
	}	
}

?>
