<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling DocmanFileVersion

require_once('plugins/docman/lib/DocmanFileVersion.php');
require_once('plugins/docman/lib/DocmanFileVersionPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'DocmanFileVersion',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'docman_file_version_id' => array('name'=>'docman_file_version_id', 'type' => 'xsd:int'),
		'docman_file_id' => array('name'=>'docman_file_id', 'type' => 'xsd:int'),
		'filename' => array('name'=>'filename', 'type' => 'xsd:string'),
		'create_date' => array('name'=>'create_date', 'type' => 'xsd:string'),
		'changes' => array('name'=>'changes', 'type' => 'xsd:string'),
		'version_number' => array('name'=>'version_number', 'type' => 'xsd:string'),
		'created_by' => array('name'=>'created_by', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfDocmanFileVersion',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:DocmanFileVersion[]')),
	'tns:DocmanFileVersion'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getDocmanFileVersions',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int'
	), 
	array('getDocmanFileVersions' => 'tns:ArrayOfDocmanFileVersion'),
	$uri, 
	$uri.'#getDocmanFileVersions',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of DocmanFileVersion
 * @param	$docman_file_id	Filter by docman_file_id
 */
function getDocmanFileVersions($sessionHash, $docman_file_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$c = new Criteria();
	
	$c->add(DocmanFileVersionPeer::DOCMAN_FILE_ID, $docman_file_id);
	$c->addAscendingOrderByColumn(DocmanFileVersionPeer::VERSION_NUMBER);
	$objs = DocmanFileVersionPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = DocmanFileVersionToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getDocmanFileVersionArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_version_id' => 'tns:ArrayOfInt',
	), 
	array('getDocmanFileVersion' => 'tns:ArrayOfDocmanFileVersion'),
	$uri, 
	$uri.'#getDocmanFileVersion',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of DocmanFileVersion
 * @param	$docman_file_version_id	Retrieve the DocmanFileVersion array identified by the array of docman_file_version_id
 */
function getDocmanFileVersionArray($sessionHash, $docman_file_version_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$ret = array();

	$crit=new Criteria();
	$crit->add(DocmanFileVersionPeer::DOCMAN_FILE_VERSION_ID,$docman_file_version_idArr,Criteria::IN);
	$crit->addAscendingOrderByColumn(DocmanFileVersionPeer::VERSION_NUMBER);
	$objs = DocmanFileVersionPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = DocmanFileVersionToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getDocmanFileVersion',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_version_id' => 'xsd:int',
	), 
	array('getDocmanFileVersion' => 'tns:DocmanFileVersion'),
	$uri, 
	$uri.'#getDocmanFileVersion',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single DocmanFileVersion
 * @param	$docman_file_version_id	Retrieve the DocmanFileVersion identified by this docman_file_version_id
 */
function getDocmanFileVersion($sessionHash, $docman_file_version_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = DocmanFileVersionPeer::retrieveByPK($docman_file_version_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getDocmanFileVersion','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return DocmanFileVersionToSOAP($obj);
}


$server->register(
	'deleteDocmanFileVersion',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_version_id' => 'xsd:int',
	), 
	array('deleteDocmanFileVersion' => 'xsd:int'),
	$uri, 
	$uri.'#deleteDocmanFileVersion',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single DocmanFileVersion
 * @param	$docman_file_version_id	Delete the DocmanFileVersion identified by this docman_file_version_id
 */
function deleteDocmanFileVersion($sessionHash, $docman_file_version_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = DocmanFileVersionPeer::retrieveByPK($docman_file_version_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteDocmanFileVersion','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteDocmanFileVersion','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

$server->register(
	'addDocmanFileVersion',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
		'changes' => 'xsd:string',
	), 
	array('addDocmanFileVersion' => 'xsd:int'),
	$uri, 
	$uri.'#getDocmanFileVersions',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a DocmanFileVersion
 * @param	$docman_file_id
 * @param	$changes
 */
function addDocmanFileVersion($sessionHash, $docman_file_id, $changes) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$docFile = DocmanFilePeer::retrieveByPK($docman_file_id);
	if (is_null($docFile) || !$docFile->userCanUpdate()) {
		return new soap_fault('', 'addDocmanFileVersion','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$docFolder = DocmanFolderPeer::retrieveByPK($docFile->getDocmanFolderId());
	if (is_null($docFolder) || !$docFolder->userCanUpdate()) {
		return new soap_fault('', 'addDocmanFileVersion','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}
	$project = ProjectPeer::retrieveByPK($docFolder->getProjectId());

	$obj = new DocmanFileVersion();
	
	$obj->setDocmanFileId( Validator::sanitizeInt($docman_file_id) );
	$obj->setChanges( Validator::sanitizeString($changes) );

	// File versions	
	$dfvs=$docFile->getDocmanFileVersions();
	$version=0;
	foreach ($dfvs as $adfv) {
		if ($adfv->getVersionNumber() > $version) {
			$version=$adfv->getVersionNumber();
		}
	}
	$version++;
	$obj->setVersionNumber($version);

	$obj->setIsApproved($project->getPermission()->check(DocmanPlugin::SECTION,0,DocmanPlugin::ADMIN));
	$obj->setCreatedBy( Session::getSession()->getUser()->getUserId() );
	try {
		$obj->save();
		return $obj->getDocmanFileVersionId();
	} catch (Exception $e) {
		return new soap_fault('', 'addDocmanFileVersion','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function GetDocmanFileVersionFilename(&$obj) {
	$name = '';
	$c = new Criteria();
	$c->add(FilesystemPeer::SECTION, DocmanFileVersion::SECTION);
	$c->add(FilesystemPeer::REF_ID, $obj->getDocmanFileVersionId());
	$frs = FilesystemPeer::doSelectOne($c);
	if ($frs != null && is_object($frs)) {
		$name = $frs->getFileName();
	}
	return $name;
}

function DocmanFileVersionToSOAP(&$obj) {
	$ret = array();
	$ret['docman_file_version_id'] = $obj->getDocmanFileVersionId();
	$ret['docman_file_id'] = $obj->getDocmanFileId();
	$ret['filename'] = GetDocmanFileVersionFilename($obj);
	$ret['create_date'] = $obj->getCreateDate();
	$ret['changes'] = $obj->getChanges();
	$ret['version_number'] = $obj->getVersionNumber();
	$ret['created_by'] = $obj->getCreatedBy();
	return $ret;
}
?>
