<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling ScmModule

require_once('plugins/scm/lib/ScmModule.php');
require_once('plugins/scm/lib/ScmModulePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'ScmModule',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'scm_module_id' => array('name'=>'scm_module_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'path' => array('name'=>'path', 'type' => 'xsd:string'),
		'publish_to_www' => array('name'=>'publish_to_www', 'type' => 'xsd:boolean'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfScmModule',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:ScmModule[]')),
	'tns:ScmModule'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getScmModules',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
	), 
	array('getScmModules' => 'tns:ArrayOfScmModule'),
	$uri, 
	$uri.'#getScmModules',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of ScmModule
 * @param	$project_id	Filter by project_id
 */
function getScmModules($sessionHash, $project_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$PrjObj = ProjectPeer::retrieveByPK($project_id);
	if (is_null($PrjObj) || !$PrjObj->userCanRead()) {
		return new soap_fault('', 'getScmModules','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$crit = new Criteria();
	$crit->add(ScmModulePeer::PROJECT_ID, $project_id);
	
	$objs = ScmModulePeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = ScmModuleToSOAP($obj);
	}
	return $res;
}

$server->register(
	'getScmModuleArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'scm_module_id' => 'ArrayOfxsd:int',
	), 
	array('getScmModule' => 'tns:ArrayOfScmModule'),
	$uri, 
	$uri.'#getScmModule',
	'rpc',
	'encoded'
);

$server->register(
	'getScmModule',
	array(
		'sessionHash'	=> 'xsd:string',
		'scm_module_id' => 'xsd:int',
	), 
	array('getScmModule' => 'tns:ScmModule'),
	$uri, 
	$uri.'#getScmModule',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single ScmModule
 * @param	$scm_module_id	Retrieve the ScmModule identified by this scm_module_id
 */
function getScmModule($sessionHash, $scm_module_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = ScmModulePeer::retrieveByPK($scm_module_id);
	if (is_null($obj)) {
		return null;
	}

	$PrjObj = ProjectPeer::retrieveByPK($obj->getProjectId());
	if (is_null($PrjObj) || !$PrjObj->userCanRead()) {
		return new soap_fault('', 'getScmModule','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return ScmModuleToSOAP($obj);
}


$server->register(
	'deleteScmModule',
	array(
		'sessionHash'	=> 'xsd:string',
		'scm_module_id' => 'xsd:int',
	), 
	array('deleteScmModule' => 'xsd:int'),
	$uri, 
	$uri.'#deleteScmModule',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single ScmModule
 * @param	$scm_module_id	Delete the ScmModule identified by this scm_module_id
 */
function deleteScmModule($sessionHash, $scm_module_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = ScmModulePeer::retrieveByPK($scm_module_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteScmModule','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot Delete This Record');
	}

	$PrjObj = ProjectPeer::retrieveByPK($obj->getProjectId());
	if (is_null($PrjObj) || !$PrjObj->userCanRead()) {
		return new soap_fault('', 'deleteScmModule','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot Delete This Record');
	}

	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteScmModule','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateScmModule',
	array(
		'sessionHash'	=> 'xsd:string',
		'scm_module_id' => 'xsd:int',
		'path' => 'xsd:string',
		'publish_to_www' => 'xsd:boolean',
	), 
	array('updateScmModule' => 'xsd:int'),
	$uri, 
	$uri.'#updateScmModule',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update ScmModule
 * @param	$scm_module_id	 ScmModule to update
 * @param	$path
 * @param	$publish_to_www
 */
function updateScmModule($sessionHash, $scm_module_id, $path, $publish_to_www) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$obj = ScmModulePeer::retrieveByPK($scm_module_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateScmModule', 'Couldn\'t update ScmModule: invalid scm_module_id '.$scm_module_id);
	}
	
	$PrjObj = ProjectPeer::retrieveByPK($obj->getProjectId());
	if (is_null($PrjObj) || !$PrjObj->userCanUpdate()) {
		return new soap_fault('', 'updateScmModule','Permission Denied - Cannot Update This Record','Permission Denied - Cannot Update This Record');
	}

	if (!is_null($path)) {
		$obj->setPath( Validator::sanitizeString($path) );
	}
	if (!is_null($publish_to_www)) {
		$obj->setPublishToWww( Validator::sanitizeString($publish_to_www) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateScmModule','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addScmModule',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'path' => 'xsd:string',
		'publish_to_www' => 'xsd:boolean',
	), 
	array('addScmModule' => 'xsd:int'),
	$uri, 
	$uri.'#addScmModule',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a ScmModule
 * @param	$project_id
 * @param	$path
 * @param	$publish_to_www
 */
function addScmModule($sessionHash, $project_id, $path, $publish_to_www) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($sessionHash, $ip);

	$PrjObj = ProjectPeer::retrieveByPK($project_id);
	if (is_null($PrjObj) || !$PrjObj->userCanUpdate()) {
		return new soap_fault('', 'addScmModule','Permission Denied - Cannot Add This Record','Permission Denied - Cannot Add This Record');
	}

	$obj = new ScmModule();
	$obj->setProjectId( Validator::sanitizeInt($project_id) );
	$obj->setPath( Validator::sanitizeString($path) );
	$obj->setPublishToWww( Validator::sanitizeString($publish_to_www) );
	try {
		$obj->save();
		return $obj->getPrimaryKey();
	} catch (Exception $e) {
		return new soap_fault('', 'addScmModule','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function ScmModuleToSOAP(&$obj) {
	$ret = array();
	$ret['scm_module_id'] = $obj->getScmModuleId();
	$ret['project_id'] = $obj->getProjectId();
	$ret['path'] = $obj->getPath();
	$ret['publish_to_www'] = $obj->getPublishToWww();
	return $ret;
}
?>
