<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @author Ruben Gutierrez ruben@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/Validator.php');
require_once('exceptions/PermissionDeniedException.php');

require_once('plugins/tracker/lib/TrackerMyQueuePeer.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}
$User = $session->getUser();

$errors = array();
$Lang = Language::getSessLang();

// First, delete all the Queued items for this user
$crit = new Criteria();
$crit->add(TrackerMyQueuePeer::USER_ID, $User->getUserId());
try {
	TrackerMyQueuePeer::doDelete($crit);
} catch (InvalidDataException $ex) {
	$feedback = implode('<br />', $ex->getErrors());
	GFUrl::redirect($url->currentBaseURL('MyQueuePopup'), array('feedback' => $feedback));
	exit();
}

// Now get and save each tracker item in the queue
$QueuedTrackerItems = $_POST['queuedTrackerItems'];
$Order = 0;
foreach ($QueuedTrackerItems as $QueuedTrackerItem) {
	$MyQueuedTrackerItem = new TrackerMyQueue();
	$MyQueuedTrackerItem->setUserId($User->getUserId());
	$MyQueuedTrackerItem->setTrackerItemId(Validator::sanitizeInt($QueuedTrackerItem));
	$MyQueuedTrackerItem->setSortOrder($Order);
	$Order++;
	try {
		$MyQueuedTrackerItem->save();
	} catch (InvalidDataException $ex) {
		$feedback = implode('<br />', $ex->getErrors());
		GFUrl::redirect($url->currentBaseURL('MyQueuePopup'), array('feedback' => $feedback));
		exit();
	}
}

$feedback = $Lang->getText('TrackerMyQueue.QueueUpdated');

if (isset($_POST['returnToPopup']) && $_POST['returnToPopup'] == 'true') {
	GFUrl::redirect($url->currentBaseURL('MyQueuePopup'), array('feedback' => $feedback));
} else {
	GFUrl::redirect($url->currentBaseURL(''), array('feedback' => $feedback));
}
?>